using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using OPC.Common;
using OPC.Data.Interface;
using OPC.Data; 


namespace CSdotNetSample
{
    public partial class Form1 : Form
    {

        // All OPC Automation Arrays start with 1 

        private OpcServer TheSrv;
        private OpcGroup TheGrp;
        private OPCItemDef[] ItemDefs = new OPCItemDef[2];
        private int[] HandlesSrv = new int[2];

        OPCItemResult[] rItm;

        const string ServerProgID = "S7200.OPCServer";
        // ProgID of OPC server 
        const string ItemA = "2:161.218.182.132:1000:1000,MB0,BYTE,RW";
        // fully qualified ID 
        const string ItemB = "2:161.218.182.132:1000:1000,VW4,WORD,RW";
        // fully qualified ID 

        public Form1()
        {
            InitializeComponent();
        }

        private void cmdConnect_Click(object sender, EventArgs e)
        {
            try
            {
                // deactivated for debugging 

                TheSrv = new OpcServer();
                TheSrv.Connect(ServerProgID);

                // Set Button Enable 
                cmdConnect.Enabled = false;
                cmdDisconnect.Enabled = true;

                cmdAddGroup.Enabled = true;
            }
            catch (Exception ex)
            {
                // Show friendly error message. 
                MessageBox.Show(ex.ToString(), "Error");
            }
        }

        private void cmdDisconnect_Click(object sender, EventArgs e)
        {
            // Set Button Enable 
            cmdDisconnect.Enabled = false;
            cmdAddGroup.Enabled = false;
            cmdConnect.Enabled = true;

            if ((TheSrv != null))
            {

                int[] aE = new int[2];

                try
                {
                    TheSrv.Disconnect();
                }
                catch (Exception ex)
                {
                    // Show friendly error message. 
                    MessageBox.Show(ex.ToString(), "Error");
                }

                TheSrv = null;

            }

        }

        private void cmdAddGroup_Click(object sender, EventArgs e)
        {
            // add our only working group 
            TheGrp = TheSrv.AddGroup(txtGroup.Text, false, 900);

            if (CheckGroupActive.Checked)
            {
                // asynch read our two items 
                TheGrp.SetEnable(true);
                TheGrp.Active = true;
            }
            else
            {
                // synch read our two items 
                TheGrp.SetEnable(true);
                TheGrp.Active = false;
            }

            TheGrp.DataChanged += theGrp_DataChange;
            TheGrp.ReadCompleted += theGrp_ReadComplete;
            TheGrp.WriteCompleted += theGrp_WriteComplete;

            // Set Button Enable 
            cmdAddGroup.Enabled = false;
            cmdDisconnect.Enabled = false;
            cmdRemGroup.Enabled = true;

            cmdAddItem.Enabled = true;

        }

        private void cmdRemGroup_Click(object sender, EventArgs e)
        {
            if ((TheGrp != null))
            {
                // 

                int[] aE = new int[2];

                try
                {
                    TheGrp.DataChanged -= theGrp_DataChange;
                    TheGrp.ReadCompleted -= theGrp_ReadComplete;
                    TheGrp.WriteCompleted -= theGrp_WriteComplete;

                    TheGrp.Remove(false);
                }
                catch (Exception ex)
                {
                    // Show friendly error message. 
                    MessageBox.Show(ex.ToString(), "Error");
                }

                TheGrp = null;
            }

            // Set Button Enable 
            cmdRemGroup.Enabled = false;
            cmdAddItem.Enabled = false;
            cmdAddGroup.Enabled = true;
            cmdDisconnect.Enabled = true;

        }

        private void CheckGroupActive_CheckedChanged(object sender, EventArgs e)
        {
            if ((TheGrp != null))
            {
                if (CheckGroupActive.CheckState == CheckState.Checked)
                {
                    // asynch read our three items 
                    TheGrp.SetEnable(true);
                    TheGrp.Active = true;
                }
                else
                {
                    // synch read our three items 
                    TheGrp.SetEnable(true);
                    TheGrp.Active = false;
                }
            }
        }

        private void cmdAddItem_Click(object sender, EventArgs e)
        {
            // add two items and save server handles 
            ItemDefs[0] = new OPCItemDef(txtItem1.Text, true, 1234, VarEnum.VT_EMPTY);
            ItemDefs[1] = new OPCItemDef(txtItem2.Text, true, 5678, VarEnum.VT_EMPTY);

            TheGrp.AddItems(ItemDefs, out rItm);
            if (rItm == null) return;


            if ( (rItm[0].Error > 0 ) | (rItm[1].Error > 0) )
            {
                // Show friendly error message. 
                MessageBox.Show("AddItems - some failed", "Error"); 

                cmdConnect.Enabled = false;
                cmdDisconnect.Enabled = true;
                cmdAddGroup.Enabled = false;
                cmdRemGroup.Enabled = true;
                cmdAddItem.Enabled = true;
                cmdRemItem.Enabled = false;

                return;
            }

            HandlesSrv[0] = rItm[0].HandleServer;
            HandlesSrv[1] = rItm[1].HandleServer;

            // Set Button Enable 
            cmdAddItem.Enabled = false;
            cmdRemGroup.Enabled = false;
            cmdRemItem.Enabled = true;
            cmdWriteSync.Enabled = true;
            cmdWriteAsync.Enabled = true;
            cmdReadSync.Enabled = true;
            cmdReadAsync.Enabled = true; 

        }

        private void cmdRemItem_Click(object sender, EventArgs e)
        {
            int[] aE = new int[2];

            if ((TheGrp != null))
            {
                TheGrp.RemoveItems(HandlesSrv, out aE);
            }

            // Set Button Enable 
            cmdRemItem.Enabled = false;
            cmdWriteSync.Enabled = false;
            cmdWriteAsync.Enabled = false;
            cmdReadSync.Enabled = false;
            cmdReadAsync.Enabled = false;
            cmdAddItem.Enabled = true;
            cmdRemGroup.Enabled = true; 

        }

        private void cmdWriteSync_Click(object sender, EventArgs e)
        {
            // Sync Write 
            int[] arrHSrv = new int[2];
            object[] arrVal = new object[2];
            int[] arrErr = new int[2];

            arrHSrv[0] = rItm[0].HandleServer;
            arrHSrv[1] = rItm[1].HandleServer;

            arrVal[0] = txtWriteVal1.Text;
            arrVal[1] = txtWriteVal2.Text;

            try
            {
                TheGrp.SyncWrite(arrHSrv, arrVal, out arrErr);

                // Show friendly error message. 
                if (arrErr[0] > 0)
                    MessageBox.Show("Item 1 FAILED. Error Code = " + arrErr[0].ToString(), "Error");
                // Show friendly error message. 
                if (arrErr[1] > 0)
                    MessageBox.Show("Item 2 FAILED. Error Code = " + arrErr[1].ToString(), "Error"); 
            }
            catch (Exception ex)
            {
                // Show friendly error message. 
                MessageBox.Show(ex.ToString(), "Error"); 
            } 

        }

        private void cmdWriteAsync_Click(object sender, EventArgs e)
        {
            int CancelID = 0;
            int[] aE = new int[2];
            // asynch write 
            object[] ItemValues = new object[2];

            ItemValues[0] = txtWriteVal1.Text;
            ItemValues[1] = txtWriteVal2.Text;

            try
            {
                TheGrp.AsyncWrite(HandlesSrv, ItemValues, 55667788, out CancelID, out aE);
            }
            catch (Exception ex)
            {
                // Show friendly error message. 
                MessageBox.Show(ex.ToString(), "Error");
            } 
        }

        private void cmdReadSync_Click(object sender, EventArgs e)
        {
            int[] aE = new int[2];

            _txtReadVal_0.Text = "";
            _txtReadVal_1.Text = "";

            // Sync Read 
            int[] arrHSrv = new int[2];
            OPCItemState[] arrStat = null;

            arrHSrv[0] = rItm[0].HandleServer;
            arrHSrv[1] = rItm[1].HandleServer;

            try
            {
                TheGrp.SyncRead(OPCDATASOURCE.OPC_DS_DEVICE, arrHSrv, out arrStat);

                if (arrStat[0].Quality == 192)
                {
                    _txtReadVal_0.Text = arrStat[0].DataValue.ToString();
                    _txtReadVal_0.BackColor = Color.White;
                }
                else
                {
                    _txtReadVal_0.Text = GetQualityText(arrStat[0].Quality);
                    _txtReadVal_0.BackColor = Color.Red;
                }

                if (arrStat[1].Quality == 192)
                {
                    _txtReadVal_1.Text = arrStat[1].DataValue.ToString();
                    _txtReadVal_1.BackColor = Color.White;
                }
                else
                {
                    _txtReadVal_1.Text = GetQualityText(arrStat[1].Quality);
                    _txtReadVal_1.BackColor = Color.Red;
                }
            }
            catch (Exception ex)
            {
                // Show friendly error message. 
                MessageBox.Show(ex.ToString(), "Error");
            } 

        }

        private void cmdReadAsync_Click(object sender, EventArgs e)
        {
            int CancelID = 0;
            int[] aE = new int[2];

            try
            {
                TheGrp.AsyncRead(HandlesSrv, 55667788, out CancelID, out aE);
            }
            catch (Exception ex)
            {
                // Show friendly error message. 
                MessageBox.Show(ex.ToString(), "Error");
            } 

        }

        private void cmdExit_Click(object sender, EventArgs e)
        {
            int[] aE = new int[2];

            if ((ItemDefs[0] != null) | (ItemDefs[1] != null))
            {
                if ((TheGrp != null))
                {
                    TheGrp.RemoveItems(HandlesSrv, out aE);
                }
            }

            if ((TheGrp != null))
            {

                try
                {
                    TheGrp.DataChanged -= theGrp_DataChange;
                    TheGrp.ReadCompleted -= theGrp_ReadComplete;
                    TheGrp.WriteCompleted -= theGrp_WriteComplete;

                    TheGrp.Remove(false);
                }
                catch (Exception ex)
                {
                    // Show friendly error message. 
                    MessageBox.Show(ex.ToString(), "Error"); 
                }
                TheGrp = null;
            }

            if ((TheSrv != null))
            {

                try
                {
                    TheSrv.Disconnect();
                }
                catch (Exception ex)
                {
                    // Show friendly error message. 
                    MessageBox.Show(ex.ToString(), "Error"); 
                }
                TheSrv = null;
            }

            this.Close(); 

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            txtItem1.Text = ItemA;
            txtItem2.Text = ItemB; 
        }

       
        private string GetQualityText( short Quality)
        {
            string functionReturnValue = null;
            switch (Quality)
            {
                case 0:
                    functionReturnValue = "BAD";
                    break;
                case 64:
                    functionReturnValue = "UNCERTAIN";
                    break;
                case 192:
                    functionReturnValue = "GOOD";
                    break;
                case 8:
                    functionReturnValue = "NOT_CONNECTED";
                    break;
                case 13:
                    functionReturnValue = "DEVICE_FAILURE";
                    break;
                case 16:
                    functionReturnValue = "SENSOR_FAILURE";
                    break;
                case 20:
                    functionReturnValue = "LAST_KNOWN";
                    break;
                case 24:
                    functionReturnValue = "COMM_FAILURE";
                    break;
                case 28:
                    functionReturnValue = "OUT_OF_SERVICE";
                    break;
                case 132:
                    functionReturnValue = "LAST_USABLE";
                    break;
                case 144:
                    functionReturnValue = "SENSOR_CAL";
                    break;
                case 148:
                    functionReturnValue = "EGU_EXCEEDED";
                    break;
                case 152:
                    functionReturnValue = "SUB_NORMAL";
                    break;
                case 216:
                    functionReturnValue = "LOCAL_OVERRIDE";

                    break;
                default:
                    functionReturnValue = "UNKNOWN QUALITY";
                    break;
            }
            return functionReturnValue;
        }
        // Callback from OnDataChange 
        public void theGrp_DataChange(object source, DataChangeEventArgs e)
        {
           // Debug.Print("DataChange event: " + e.transactionID.ToString());

            double dVal = Convert.ToDouble(TxtDataChange.Text) + 1;
            SetText(ref TxtDataChange, dVal.ToString());

            //OPCItemState s; // = default(OPCItemState);
            foreach ( OPCItemState s in e.sts) //(  s in  e.sts)
            {
                if (s.Error > 0)
                {
                    if (s.HandleClient == 1234)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 1" + " FAILED. Error Code = " + s.Error.ToString(), "Error");
                    }
                    else if (s.HandleClient == 5678)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 2" + " FAILED. Error Code = " + s.Error.ToString(), "Error");
                    }
                }
                else
                {
                    if (s.HandleClient == 1234)
                    {
                        if (s.Quality == 192)
                        {
                            if ((s.DataValue != null))
                            {
                                SetText(ref _txtChangeVal_0, s.DataValue.ToString());
                                _txtChangeVal_0.BackColor = Color.White;
                            }
                        }
                        else
                        {
                            SetText(ref _txtChangeVal_0, GetQualityText(s.Quality));
                            _txtChangeVal_0.BackColor = Color.Red;
                        }
                    }
                    else if (s.HandleClient == 5678)
                    {
                        if (s.Quality == 192)
                        {
                            if ((s.DataValue != null))
                            {
                                SetText(ref _txtChangeVal_1, s.DataValue.ToString());
                                _txtChangeVal_1.BackColor = Color.White;
                            }
                        }
                        else
                        {
                            SetText(ref _txtChangeVal_1, GetQualityText(s.Quality));
                            _txtChangeVal_1.BackColor = Color.Red;
                        }
                    }
                }
            }
        }

        public delegate void SetTextCallback(ref TextBox textBox, string text);

        private void SetText(ref TextBox textBox, string text)
        {

            // InvokeRequired required compares the thread ID of the 
            // calling thread to the thread ID of the creating thread. 
            // If these threads are different, it returns true. 
            if (TxtDataChange.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetText);
                this.Invoke(d, new object[] { textBox, text });
            }
            else
            {
                textBox.Text = text;
            }
        }

        // Callback from AsyncRead 
        public void theGrp_ReadComplete(object source, ReadCompleteEventArgs e)
        {
            // Console.WriteLine("ReadComplete event: " + e.transactionID.ToString()) 

            double dVal = Convert.ToDouble(TxtAReadComplete.Text) + 1;
            SetText(ref TxtAReadComplete, dVal.ToString());

            //OPCItemState s; // = default(OPCItemState);
            foreach ( OPCItemState s in e.sts)
            {
                if (s.Error > 0 )
                {
                    if (s.HandleClient == 1234)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 1" + " FAILED. Error Code = " + s.Error.ToString(), "Error");
                    }
                    else if (s.HandleClient == 5678)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 2" + " FAILED. Error Code = " + s.Error.ToString(), "Error");
                    }
                }
                else
                {
                    if (s.HandleClient == 1234)
                    {
                        if (s.Quality == 192)
                        {
                            if ((s.DataValue != null))
                            {
                                SetText(ref _txtReadVal_0, s.DataValue.ToString());
                                _txtReadVal_0.BackColor = Color.White;
                            }
                        }
                        else
                        {
                            SetText(ref _txtReadVal_0, GetQualityText(s.Quality));
                            _txtReadVal_0.BackColor = Color.Red;
                        }
                    }
                    else if (s.HandleClient == 5678)
                    {
                        if (s.Quality == 192)
                        {
                            if ((s.DataValue != null))
                            {
                                SetText(ref _txtReadVal_1, s.DataValue.ToString());
                                _txtReadVal_1.BackColor = Color.White;
                            }
                        }
                        else
                        {
                            SetText(ref _txtReadVal_1, GetQualityText(s.Quality));
                            _txtReadVal_1.BackColor = Color.Red;
                        }
                    }
                }
            }
        }
        // Callback from AsyncWrite 
        public void theGrp_WriteComplete(object source, WriteCompleteEventArgs e)
        {
            // Console.WriteLine("WriteComplete event: " + e.transactionID.ToString()) 
            double dVal = Convert.ToDouble(TxtAWriteComplete.Text) + 1;
            SetText(ref TxtAWriteComplete, dVal.ToString());

            //OPCWriteResult r = default(OPCWriteResult);
            foreach (OPCWriteResult r in e.res)
            {
                if (r.Error > 0)
                {
                    if (r.HandleClient == 1234)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 1" + " FAILED. Error Code = " + r.Error.ToString(), "Error");
                    }
                    else if (r.HandleClient == 5678)
                    {
                        // Show friendly error message. 
                        MessageBox.Show("Item 2" + " FAILED. Error Code = " + r.Error.ToString(), "Error");
                    }
                }
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (cmdRemItem.Enabled == true) 
                cmdRemItem_Click(cmdRemItem, new System.EventArgs());
            
            if (cmdRemGroup.Enabled == true) 
                cmdRemGroup_Click(cmdRemGroup, new System.EventArgs());
            
            if (cmdDisconnect.Enabled == true) 
                cmdDisconnect_Click(cmdDisconnect, new System.EventArgs()); 
        } 
    }
}