﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Remoting;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;

namespace TapfloDB
{
    public static class Helper
    {
        private static Dictionary<string, View.AbstractView> views = new Dictionary<string, View.AbstractView>(3);
        private static String label = "TapfloDB - "; // String.Format("TapfloDB {0} - ", Server.AppVersion);

        public static View.AbstractView CreateView(string view){
            return CreateView(view, null);
        }
        public static View.AbstractView CreateView(string view, string assembly){
            View.AbstractView v = null;
            if (views.TryGetValue(view, out v)) 
                return v;

            try {
                ObjectHandle oh = Activator.CreateInstance(assembly, "TapfloDB.View." + view + ".Main");
                v = oh.Unwrap() as View.AbstractView;
                v.Anchor = AnchorStyles.Top | AnchorStyles.Left;
                v.Dock = DockStyle.Fill;
                v.Text = label + v.Text; // set view label
                
                views.Add(view, v);
            }
            catch (Exception exp){
                Server.Log("Błąd tworzenia widoku {0} - {1}", v, exp.Message);
                if (exp.InnerException != null) {
                    Server.Log("# {0}\n{1}\n", exp.InnerException.Message, exp.InnerException.StackTrace);
                }
                MessageBox.Show("Wybrana funkcja jest niedostępna!", "Funkcja niedostępna", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }

            return v;
        }

        public static string SHA256(this string password){
            SHA256Managed crypt = new SHA256Managed();
            string hash = String.Empty;
            byte[] crypto = crypt.ComputeHash(Encoding.UTF8.GetBytes(password), 0, Encoding.UTF8.GetByteCount(password));
            foreach (byte bit in crypto){
                hash += bit.ToString("x2");
            }

            return hash;
        }

        public static bool IsSet(this Enum input, Enum matchTo){
            return (Convert.ToUInt32(input) & Convert.ToUInt32(matchTo)) != 0;
        }
    }

    [Flags]
    public enum FormMode : int {
        Add = 1,
        Edit = 2,
        Change = 3
    };
}
