﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TapfloDB.UI
{
    public partial class LoginBox : UserControl
    {
        // Border style, color and width
        private LoginBoxMode mode = LoginBoxMode.Normal;
        private ButtonBorderStyle borderStyle = ButtonBorderStyle.Solid;
        private Color borderColor = Color.Gray;
        private int borderWidth = 1;
        
        public LoginBox(){
            InitializeComponent();
            this.Paint += new PaintEventHandler(LoginBox_Paint);
            this.Resize += new EventHandler(LoginBox_Resize);
        }

        #region New Props

        public LoginBoxMode BorderMode
        {
            get { return mode; }
            set { 
                if(mode == value)
                    return;
 
                mode = value;
                if(mode == LoginBoxMode.Undefined) { }
                else if (mode == LoginBoxMode.Normal){
                    borderStyle = ButtonBorderStyle.Solid;
                    borderColor = Color.Gray;
                    borderWidth = 1;
                }
                else {
                    borderColor = Color.Red;
                    borderWidth = 2;
                    if (mode == LoginBoxMode.RedDashed) borderStyle = ButtonBorderStyle.Dashed;
                    else if (mode == LoginBoxMode.RedDotted) borderStyle = ButtonBorderStyle.Dotted;
                    else if (mode == LoginBoxMode.RedSolid) borderStyle = ButtonBorderStyle.Solid;
                }

                Invalidate(); 
            }
        }
        
        public new ButtonBorderStyle BorderStyle
        {
            get { return borderStyle; }
            set { borderStyle = value; Invalidate(); }
        }
        public Color BorderColor
        {
            get { return borderColor; }
            set { borderColor = value; Invalidate(); }
        }
        public int BorderWidth
        {
            get { return borderWidth; }
            set {
                int val = value < 1 ? 1 : value;
                val = val > 3 ? 3 : val;
                if (borderWidth == val)
                    return;

                borderWidth = val;
                Invalidate(); 
            }
        }

        #endregion

        #region Exposed properties of TextBox

        public override string Text
        {
            get { return textBox.Text; }
            set { textBox.Text = value; }
        }
        public new Size Size
        {
            get { return base.Size; }
            set { base.Size = value; InvalidateSize(); }
        }
        public int MaxLength
        {
            get { return textBox.MaxLength; }
            set { textBox.MaxLength = value; }
        }

        #endregion

        #region Exposed events

        public event EventHandler TextBoxClick
        {
            add { textBox.Click += value; }
            remove { textBox.Click -= value; }
        }
        public new event EventHandler TextChanged
        {
            add { textBox.TextChanged += value; }
            remove { textBox.TextChanged -= value; }
        }

        #endregion

        #region Adjustable Border 

        private void LoginBox_Resize(object sender, EventArgs e){
            InvalidateSize();
        }

        private void LoginBox_Paint(object sender, PaintEventArgs e){
            ControlPaint.DrawBorder(e.Graphics, this.ClientRectangle, 
                borderColor, borderWidth, borderStyle, //Left
                borderColor, borderWidth, borderStyle, //Top
                borderColor, borderWidth, borderStyle, //Right
                borderColor, borderWidth, borderStyle  //Bottom
            );
        }

        private void InvalidateSize(){
            textBox.Size = new Size(this.Width - 6, this.Height);
            textBox.Location = new Point(3, 3);
        }

        #endregion
    }

    public enum LoginBoxMode {
        Undefined = 0,
        Normal = 1,
        RedSolid = 2,
        RedDotted = 3,
        RedDashed = 4
    }
}
