﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Npgsql;

namespace TapfloDB.View.Faults
{
    public partial class Main : AbstractView
    {
        private DataTable dt = new DataTable();
        private NpgsqlDataAdapter da;

        public Main(){
            InitializeComponent();
            Text = "Lista Defektów";
        }

        private void Main_Load(object sender, EventArgs e){
            Server.Connect();
            da = Server.DataAdapter("SELECT id, gid, name, description FROM tdb.\"TestFailReason\" ORDER BY id;");
            
            da.Fill(dt);
            dgv.DataSource = dt;
        }

        private void dgv_CellValueChanged(object sender, DataGridViewCellEventArgs e){
            if (e.RowIndex < 0 || e.ColumnIndex < 2) return;
            NpgsqlCommand cmd;

            // Add
            object id = dgv[0, e.RowIndex].Value;
            if (id == null || id is DBNull){
                object name = dgv[2, e.RowIndex].Value;
                if (name == null || name is DBNull)
                    return;

                try {
                    cmd = Server.Command("INSERT INTO tdb.\"TestFailReason\" (gid, name, description) VALUES(0, :name, :description);select currval('tdb.\"TestFailReason_id_seq\"');");
                    cmd.Parameters.AddWithValue("name", dgv[2, e.RowIndex].Value);
                    cmd.Parameters.AddWithValue("description", dgv[3, e.RowIndex].Value);
                    id = cmd.ExecuteScalar();
                    if (id != null && (long)id > 0){
                        dgv[0, e.RowIndex].Value = id; // set id to allow deleting
                        Session.Curr.Action("Dodano nowy Defekt {0}", dgv[2, e.RowIndex].Value);
                        Fault.Add((int)((long)id), (string)dgv[2, e.RowIndex].Value);
                    }
                }
                catch (Exception exp){
                    Server.Log("Błąd dodawania Defektu - {0}", exp.Message);
                    MessageBox.Show("Podczas dodawania nowego Defektu wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd dodawania Defektu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
            else { // Update
                try {
                    cmd = Server.Command("UPDATE tdb.\"TestFailReason\" SET name = :name, description = :description WHERE id = :id;");
                    cmd.Parameters.AddWithValue("id", id);
                    cmd.Parameters.AddWithValue("name", dgv[2, e.RowIndex].Value);
                    cmd.Parameters.AddWithValue("description", dgv[3, e.RowIndex].Value);
                    if (cmd.ExecuteNonQuery() > 0)
                        Session.Curr.Action("Zmodyfikowano Defekt {0}", dgv[2, e.RowIndex].Value);
                }
                catch (Exception exp){
                    Server.Log("Błąd modyfikacji Defektu #{0} - {1}", id, exp.Message);
                    MessageBox.Show("Podczas próby modyfikacji Defektu wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd modyfikacji Defektu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
        }

        private bool remove(DataGridViewRow row){
            if (row.Cells[0].Value is DBNull) 
                return true; // Not in DB so nothing to do

            NpgsqlCommand cmd = Server.Command("DELETE FROM tdb.\"TestFailReason\" WHERE id = :id;");
            try {
                int id = (int)row.Cells[0].Value;
                cmd.Parameters.AddWithValue("id", id);
                return cmd.ExecuteNonQuery() > 0;
            }
            catch (Exception exp){
                Server.Log("Błąd usuwania Defektu #{0} - {1}", row.Cells[0].Value, exp.Message);
            }

            return false;
        }

        private void dgv_UserDeletingRow(object sender, DataGridViewRowCancelEventArgs e){
            if (dgv.Rows.Count == 0) return;

            if (remove(e.Row))
                Session.Curr.Action("Pomyślnie skasowano Defekt {0}", e.Row.Cells[2].Value);
            else
                MessageBox.Show("Podczas usuwania Defektu wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany błąd usuwania Defektu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }

        private void bdelete_Click(object sender, EventArgs e){
            int deleted = 0;
            if (dgv.SelectedRows.Count == 1){
                DataGridViewRowCancelEventArgs rce = new DataGridViewRowCancelEventArgs(dgv.SelectedRows[0]);
                dgv_UserDeletingRow(sender, rce);
                if (!rce.Cancel)
                    dgv.Rows.RemoveAt(rce.Row.Index);
            }
            else if (dgv.SelectedRows.Count > 0) {
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (remove(r)){
                        dgv.Rows.RemoveAt(r.Index);
                        deleted++;
                    }
                }

                if (deleted == dgv.SelectedRows.Count)
                    Session.Curr.Action("Pomyślnie skasowano {1} Defektów", deleted);
                else
                    Session.Curr.Action("Skasowano {0}/{1} Defektów", deleted, dgv.SelectedRows.Count);
            }
        }
    }
}
