﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Globalization;

namespace TapfloDB.View.ProductionData
{
    public partial class Export : Form
    {
        private bool addheaders = false;
        private bool quotes = false;
        private String decsep = ".";
        private String separator = ",";
        private String firstcol = "";
        private String headers = "\"Data\",\"{0}\",\"Zdane\",\"Ilość\",\"Cel\"";
        private String format;

        public int Mode { get; private set; }

        public string FileName {
            get { return save.FileName; }
        }

        public Export(int periods, int products){
            InitializeComponent();
            Mode = -1;

            if (rperiods.Enabled = periods > 0)
                rperiods.Text += " (" + periods + " przedziałów)";
            if (rproducts.Enabled = products > 0)
                rproducts.Text += " (" + products + " produktów)";
        }

        private void bsave_Click(object sender, EventArgs e){
            if (!rperiods.Checked && !rproducts.Checked) {
                MessageBox.Show("Nie wybrano Danych Produkcyjnych do eksportu.", "Nie wybrano danych do eksportu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (String.IsNullOrEmpty(tseparator.Text)) {
                MessageBox.Show("Nie podano separatora kolumn dla pliku CSV.", "Nie podano separatora kolumn", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (String.IsNullOrEmpty(tdecimal.Text)){
                MessageBox.Show("Nie podano separatora części ułamkowej.", "Nie podano separatora dziesiętnego", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            //Adjust data
            if (rperiods.Checked){
                firstcol = "Przedział";
                Mode = 1;
            }
            else if (rproducts.Checked){
                firstcol = "Produkt";
                Mode = 2;
            }

            addheaders = cheaders.Checked;
            decsep = tdecimal.Text;

            if (cquotes.Checked)
                format = "{0},\"{1}\",\"{2}\",\"{3}\",\"{4}\"";
            else
                format = "{0},\"{1}\",{2},{3},{4}";

            if (tseparator.Text != separator){
                format = format.Replace(separator, tseparator.Text);
                headers = headers.Replace(separator, tseparator.Text);
                separator = tseparator.Text;
            }

            DialogResult = save.ShowDialog();
        }

        public void Save(ProdData[] pd){
            if(!bg.IsBusy)
                bg.RunWorkerAsync(pd);
        }

        public void Cancel(){
            if (bg.IsBusy)
                bg.CancelAsync();
        }

        private void bg_DoWork(object sender, DoWorkEventArgs e){
            ProdData[] pds = (ProdData[])e.Argument;

            ProdData curr = pds[0];
            String prodday = Helper.ProdDay.ToString("dd.MM.yyyy");

            int i = 0, p = 0;
            StreamWriter sw = null;
            try {
                sw = new StreamWriter(FileName, false, Encoding.UTF8);
                if (addheaders)
                    sw.WriteLine(headers, firstcol);

                foreach(ProdData pd in pds){
                    curr = pd;
                    sw.WriteLine(format, 
                        prodday,
                        Format(pd.Parameter),
                        pd.Passed,
                        pd.Count,
                        pd.Target
                    );
                    
                    p = (int)(i * 100.0f / pds.Length);
                    if (p % 5 == 0)
                        bg.ReportProgress(p);

                    i++;
                }

                e.Result = pds.Length;
            }
            catch (ArgumentException exp){
                e.Result = String.Format("{0} dla {1}", exp.Message, curr.Parameter);
            }
            catch (Exception exp) {
                Server.Log("Przerwano exportowanie danych na {0} - {1} w {2}\n{3}", i, exp.GetType(), curr.Parameter, exp.ToString());
                e.Result = String.Format("Nieoczekiwany błąd podczas exportowania {0}",  curr.Parameter);
            }
            finally {
                if(sw != null)
                    sw.Close();
            }
        }

        private void bg_ProgressChanged(object sender, ProgressChangedEventArgs e){
            Session.Curr.Action("Zapisywanie... ("+e.ProgressPercentage+"%)");
        }

        private void bg_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e){
            if (e.Result is Int32)
                Session.Curr.Action("Pomyślnie zapisano " + e.Result + " rekordów w " + FileName);
            else {
                string msg = (e.Result is string) ? e.Result as string : "Nieznany błąd podczas Exportowania"; 
                Session.Curr.Action("Nie udalo się zapisać do pliku " + FileName);
                MessageBox.Show(msg, "Błąd podczas exportu", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private string Format(object o) {
            if(o is DateTime)
                return ((DateTime)o).ToString("HH:mm");
            return (string)o;
        }
    }
}
