﻿using Npgsql;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TapfloDB.View.ProductionData
{
    public class Helper
    {
        public static DateTime PeriodStart { get; private set; }
        public static DateTime PeriodEnd { get; private set; }
        public static TimeSpan PeriodLength { get; private set; }

        public static DateTime ProdDay { get; private set; }

        private static Dictionary<DateTime, ProdData> periods = new Dictionary<DateTime, ProdData>();
        public static BindingList<ProdData> Periods = new BindingList<ProdData>();

        public static BindingList<ProdData> Products = new BindingList<ProdData>();

        public static ProdData Total = new ProdData("Total", 0);

        public static DateTime LastUpdate { get; private set; }
        public static TimeSpan UpdatePeriod { get; set; }

        static Helper() { 
            LastUpdate = DateTime.Today.Subtract(TimeSpan.FromDays(1));
            UpdatePeriod = new TimeSpan(0, 4, 55);
        }

        public static bool Load(DateTime day){
            if (day == ProdDay)
                return false;

            ProdDay = day;
            periods.Clear();
            Periods.Clear();
            RefreshTargets(null); // load defaults

            return Refresh(true);
        }

        public static bool Refresh(bool force) {
            if (!force && DateTime.Now.Subtract(LastUpdate).TotalMinutes < 5)
                return false;

            RefreshTargets(ProdDay);
            if (CalcPeriods(ProdDay) && CalcProducts(ProdDay)){
                Session.Curr.Action("Zaktualizowano Dane Produkcyjne");
                return true;
            }

            return false;
        }

        private static void ResetPeriods() {
            ProdData pd;
            DateTime period;
            for (int i = 0; i < 24; ++i){
                period = new DateTime(1, 1, 1, i, 0, 0);
                pd = new ProdData(period, null);
                periods.Add(period, pd);
                Periods.Add(pd);
            }
        }

        private static bool RefreshTargets(DateTime? day){
            Server.Connect();
            NpgsqlDataReader dr = null;
            try {
                NpgsqlCommand cmd;
                String select = "SELECT start_time, end_time, period, target::text FROM tdb.\"ProductionData\" ";
                if (!day.HasValue)
                    cmd = Server.Command(select+"WHERE prodday IS NULL;");
                else {
                    cmd = Server.Command(select+"WHERE prodday = :day;");
                    cmd.Parameters.AddWithValue("day", day.Value);
                }
                dr = cmd.ExecuteReader();
                if (!dr.HasRows){
                    if (day.HasValue){
                        cmd = Server.Command("INSERT INTO tdb.\"ProductionData\" (prodday, start_time, end_time, period, target) SELECT :day::date, start_time, end_time, period, target FROM tdb.\"ProductionData\" WHERE prodday IS NULL;");
                        cmd.Parameters.AddWithValue("day", day.Value.ToShortDateString());
                        return cmd.ExecuteNonQuery() > 0;
                    }
                    return false;
                }
                dr.Read();
                
                PeriodStart = (DateTime)dr["start_time"];
                PeriodEnd = (DateTime)dr["end_time"];
                PeriodLength = (TimeSpan)dr["period"];

                int idx;
                DateTime curr = PeriodStart;
                foreach (int? t in PSqlArrays.ParseInt(dr["target"], 1)){
                    if (periods.ContainsKey(curr))
                        periods[curr].Target = t;
                    else if (t.HasValue){
                        ProdData pd = new ProdData(curr, t);
                        periods.Add(curr, pd);
                        idx = (int)periods.OrderBy(kp => kp.Key).Select((kp, i) => new { Idx = i, Obj = kp.Value }).First(o => o.Obj == pd).Idx;
                        Periods.Insert(idx, pd);
                    }
                    
                    curr += PeriodLength;
                }
            }
            catch (Exception exp){
                Server.Log("Błąd ładowania listy celów dla daty: {0} - {1}", day.HasValue ? day.Value.ToShortDateString() : "NULL", exp.Message);
                MessageBox.Show("Podczas ładowania Celów Produkcyjnych wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd ładowania Celów Producyjnych", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return false;
            }
            finally {
                if(dr != null)
                    dr.Close();
            }

            Session.Curr.Action("Załadowano Cele Produkcyjne dla Okresów");
            return true;
        }

        private static string fwgo = "FROM tdb.\"TestDatabase\" WHERE date(date) = :day AND nr = 1 GROUP BY 1 ORDER BY 1;";
        private static bool CalcPeriods(DateTime day){
            Server.Connect();
            NpgsqlDataReader dr = null;
            try {
                NpgsqlCommand cmd = Server.Command("SELECT extract(hour from date)::int as period, count(CASE state & 2 WHEN 2 THEN 1 ELSE null END)::int as passed, count(*)::int as tested "+fwgo);
                cmd.Parameters.AddWithValue("day", day);
                dr = cmd.ExecuteReader();

                LastUpdate = DateTime.Now;

                DateTime period;
                int idx, p, c, passed = 0, count = 0;
                while (dr.Read()){
                    period = new DateTime(1, 1, 1, (int)dr["period"], 0, 0);
                    passed += (p = (int)dr["passed"]);
                    count += (c = (int)dr["tested"]);

                    if (periods.ContainsKey(period)){
                        periods[period].Passed = p;
                        periods[period].Count = c;
                    }
                    else {
                        ProdData pd = new ProdData(period, p, c);
                        periods.Add(period, pd);
                        idx = (int)periods.OrderBy(kp => kp.Key).Select((kp,i) => new {Idx = i, Obj = kp.Value}).First(o => o.Obj == pd).Idx;
                        Periods.Insert(idx, pd);
                    }
                }

                // Update total
                Total.Passed = passed;
                Total.Count = count;
            }
            catch (Exception exp){
                Server.Log("Błąd obliczania danych dla przedział czasowych - {0}", exp.Message);
                MessageBox.Show("Podczas obliczania Danych Produkcyjnych dla Przedziałów Czasowych wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd obliczania Danych Producyjnych", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return false;
            }
            finally {
                if(dr != null)
                    dr.Close();
            }

            return true;
        }

        private static bool CalcProducts(DateTime day){
            Server.Connect();
            NpgsqlDataReader dr = null;
            try {
                NpgsqlCommand cmd = Server.Command("SELECT product, count(CASE state & 2 WHEN 2 THEN 1 ELSE null END)::int as passed, count(*)::int as tested "+fwgo);
                cmd.Parameters.AddWithValue("day", day);
                dr = cmd.ExecuteReader();

                Products.Clear();
                LastUpdate = DateTime.Now;

                String product;
                while (dr.Read()){
                    product = (string)dr["product"];

                    ProdData pd = new ProdData(product, (int)dr["passed"], (int)dr["tested"]);
                    Products.Add(pd);
                }
            }
            catch (Exception exp){
                Server.Log("Błąd obliczania danych produktowych - {0}", exp.Message);
                MessageBox.Show("Podczas obliczania Danych Produkcyjnych dla Produktów wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd obliczania Danych Producyjnych", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return false;
            }
            finally {
                if(dr != null)
                    dr.Close();
            }

            return true;
        }

        public static bool UpdateTargets() {
            var order = periods.Where(kp => kp.Value.Target.HasValue).OrderBy(kp => kp.Key);
            int count = order.Count();
            List<int?> targets = new List<int?>(count);
            DateTime min = PeriodStart, curr = PeriodEnd + PeriodLength;
            if (count > 0)
                min = curr = order.First().Key;
            for (; count > 0; curr += PeriodLength){
                if (periods.ContainsKey(curr)){
                    if (periods[curr].Target.HasValue) 
                        --count;
                    targets.Add(periods[curr].Target);
                }
                else
                    targets.Add(null);
            }

            Server.Connect();
            try {
                NpgsqlCommand cmd = Server.Command("UPDATE tdb.\"ProductionData\" SET start_time = :start::time, end_time = :end, target = :target, updated = :updated WHERE prodday IS NULL OR prodday = :day;");
                cmd.Parameters.AddWithValue("start", min.ToString("hh:mm"));
                cmd.Parameters.AddWithValue("end", curr - PeriodLength);
                cmd.Parameters.AddWithValue("updated", DateTime.Now);
                cmd.Parameters.AddWithValue("day", ProdDay);
                cmd.Parameters.Add("target", NpgsqlTypes.NpgsqlDbType.Integer | NpgsqlTypes.NpgsqlDbType.Array);
                cmd.Parameters["target"].Value = targets.ToArray();
                
                int res = cmd.ExecuteNonQuery();
                if (res > 0){
                    PeriodStart = min;
                    PeriodEnd = curr;
                    
                    Session.Curr.Action("Załadowano Cele Produkcyjne");
                    return true;
                }
            }
            catch (Exception exp){
                Server.Log("Błąd aktualizacji listy celów dla daty: {0} - {1}", ProdDay, exp.Message);
                MessageBox.Show("Podczas zapisywania Celów Producyjnych wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd zapisywania Celów Producyjnych", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }

            return false;
        }
    }
}
