﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;

namespace TapfloDB.View.ProductionData
{
    public partial class Main : AbstractView
    {
        private bool live = true;

        private DataGridViewCellStyle normal;
        private DataGridViewCellStyle highlight;

        #region Init

        public Main(){
            InitializeComponent();
            SupportExport = true;
            NewWindow = true;
            Text = "Dane Produkcyjne";

            dgvperiods.AutoGenerateColumns = false;
            dgvproducts.AutoGenerateColumns = false;

            normal = dgvperiods.DefaultCellStyle;
            highlight = new DataGridViewCellStyle(normal);
            highlight.ForeColor = Color.Red;
            highlight.Font = new Font(highlight.Font, FontStyle.Bold);
        }

        private void Main_Load(object sender, EventArgs e){
            Data.User u = Session.Curr.User;

            //Filter Menus
            if (u.Type != Users.UserType.Admin){
                catarget.ReadOnly = true;
            }

            // start load data
            Helper.Load(DateTime.Today);

            // Bind data to infopanel
            lpassed.DataBindings.Add("Text", Helper.Total, "Passed");
            Binding b = new Binding("Text", Helper.Total, "Yield");
            b.Format += lyield_Format;
            lyield.DataBindings.Add(b);

            // Set DataSources for left and right pane
            dgvperiods.DataSource = Helper.Periods;
            dgvproducts.DataSource = Helper.Products;
        }

        private void Main_ParentChanged(object sender, EventArgs e){
            if (Parent == null){
                autoupdate.Enabled = false;
            }
            else {
                Helper.Refresh(false);
                autoupdate.Enabled = true;
            }
        }


        private void lyield_Format(object sender, ConvertEventArgs e){
            double v = (double)e.Value;
            e.Value = Double.IsNaN(v) ? "0%" : v.ToString("P0");
            ldate.Text = Helper.LastUpdate.ToString("dd.MM.yyyy\nHH:mm:ss");
        }

        #endregion

        #region Date & Refresh

        private void date_ValueChanged(object sender, EventArgs e){
            autoupdate.Enabled = live = date.Value == DateTime.Now;
            Helper.Load(date.Value);
        }

        private void brefresh_Click(object sender, EventArgs e){
            autoupdate.Enabled = false;
            Helper.Refresh(true);
            autoupdate.Enabled = live;
        }

        private void autoupdate_Tick(object sender, EventArgs e){
            Helper.Refresh(false);
        }

        #endregion

        #region Info Panel

        private void tlpinfo_Resize(object sender, EventArgs e){
            SizeF max = new SizeF(tlpinfo.Width / 3 - 20, tlpinfo.Height - 50); // first row height + margin
            if (max.Width < 0 || max.Height < 0) {
                Screen curr = Screen.FromControl(this);
                Server.Log("name: {0}{1}; res: {2}x{3}; area: {4}x{5}", curr.DeviceName, curr.Primary ? " (Primary)" : "", curr.Bounds.Width, curr.Bounds.Height, curr.WorkingArea.Width, curr.WorkingArea.Height);
                Server.Log("window: {0}x{1}; control: {2}x{3}", (Parent != null ? Parent.Width : -1), (Parent != null ? Parent.Height : -1), Width, Height);
                Server.Log("tlp: {0}x{1}; max: {2}x{3}", tlpinfo.Width, tlpinfo.Height, max.Width, max.Height);
                return;
            }
            using (Graphics g = tlpinfo.CreateGraphics()) {
                // Yield & Count
                SizeF sz = g.MeasureString("100%", lyield.Font);
                float h = Math.Min(sz.Height / sz.Width, max.Height / max.Width) * max.Width;
                float emSize = (float)Math.Floor(h * lyield.Font.Size / sz.Height);
                lpassed.Font = lyield.Font = new Font(lyield.Font.Name, emSize);
                
                // Date
                sz = g.MeasureString(ldate.Text, ldate.Font);
                h = 0.75f * Math.Min(sz.Height / sz.Width, max.Height / max.Width) * max.Width;
                emSize = (float)Math.Floor(h * ldate.Font.Size / sz.Height);
                ldate.Font = new Font(ldate.Font.Name, emSize);
            }
        }

        #endregion

        #region DGVs

        private void dgvperiods_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e){
            if (e.RowIndex < 0) return;

            ProdData pd = dgvperiods.Rows[e.RowIndex].DataBoundItem as ProdData;
            if (pd == null) return;
            
            if (e.ColumnIndex == 0) {
                DateTime dt = (DateTime)e.Value;
                e.Value = dt.ToShortTimeString();
            }
            else if (e.ColumnIndex == 2) {
                if (live && pd.Count > 0 && pd.Count < pd.Target)
                    dgvperiods.Rows[e.RowIndex].DefaultCellStyle = highlight;
                else
                    dgvperiods.Rows[e.RowIndex].DefaultCellStyle = normal;
            }
        }

        private void dgvperiods_CellValueChanged(object sender, DataGridViewCellEventArgs e){
            if (e.RowIndex < 0 || e.ColumnIndex != 3) return;
            Helper.UpdateTargets();
        }

        #endregion

        private void bexport_Click(object sender, EventArgs e){
            Export();
        }

        public override void Export(){
            Export e = new Export(Helper.Periods.Count, Helper.Products.Count);
            if (e.ShowDialog() == DialogResult.OK) {
                if (e.Mode == 1)
                    e.Save((from p in Helper.Periods orderby p.Parameter select p).ToArray());
                else if (e.Mode == 2)
                    e.Save((from p in Helper.Products orderby p.Parameter select p).ToArray());
            }
        }
    }
}
