﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace TapfloDB.View.ProductionData
{
    public class ProdData : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName){
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

        protected bool SetField<T>(ref T field, T value, string propertyName){
            if (EqualityComparer<T>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propertyName);
            return true;
        }

        #endregion

        #region Properties

        public object Parameter { get; private set; }

        private int passed = 0;
        public int Passed {
            get { return passed; }
            set { SetField(ref passed, value, "Passed"); OnPropertyChanged("Yield"); }
        }

        private int count = 0;
        public int Count {
            get { return count; }
            set { SetField(ref count, value, "Count"); OnPropertyChanged("Yield"); }
        }

        private int? target;
        public int? Target {
            get { return target; }
            set { SetField(ref target, value, "Target"); }
        }

        public double Yield {
            get { return passed / (float)count; }
        }

        #endregion

        public ProdData(object param, int? target) {
            Parameter = param;
            Target = target;
        }

        public ProdData(object param, int passed, int count) {
            Parameter = param;
            Passed = passed;
            Count = count;
            Target = null;
        }
    }
}
