﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Globalization;

namespace TapfloDB.View.TDB
{
    public partial class Export : Form
    {
        private bool headers = false;
        private bool quotes = false;
        private String decsep = ".";
        private String separator = ",";
        private String format = "\"{0}\",\"{1}\",{2},\"{3}\",{4},{5},\"{6}\",{7}";
        private Dictionary<int, string> names;

        public int Mode { get; private set; }

        public string FileName {
            get { return save.FileName; }
        }

        public Export(int selected, int search, int all){
            InitializeComponent();
            Mode = -1;

            if (rselected.Enabled = selected > 0)
                rselected.Text += " (" + selected + " wierszy)";
            if (rsearch.Enabled = search > 0)
                rsearch.Text += " (" + search + " wierszy)";
            if(rall.Enabled = all > 0)
                rall.Text += " (" + all + " wierszy)";
        }

        private void bsave_Click(object sender, EventArgs e){
            if (!rselected.Checked && !rsearch.Checked && !rall.Checked) {
                MessageBox.Show("Nie wybrano zakresu eksportu danych z Test Database.", "Nie wybrano zakresu eksportu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (String.IsNullOrEmpty(tseparator.Text)) {
                MessageBox.Show("Nie podano separatora kolumn dla pliku CSV.", "Nie podano separatora kolumn", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (String.IsNullOrEmpty(tdecimal.Text)){
                MessageBox.Show("Nie podano separatora części ułamkowej.", "Nie podano separatora dziesiętnego", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            //Adjust data
            if (rselected.Checked) Mode = 1;
            else if (rsearch.Checked) Mode = 2;
            else if (rall.Checked) Mode = 3;

            headers = cheaders.Checked;
            decsep = tdecimal.Text;
            quotes = cquotes.Checked;

            if (tseparator.Text != separator){
                format = format.Replace(separator, tseparator.Text);
                separator = tseparator.Text;
            }

            DialogResult = save.ShowDialog();
        }

        public void Save(ICollection dr){
            if(!bg.IsBusy)
                bg.RunWorkerAsync(dr);
        }

        public void Cancel(){
            if (bg.IsBusy)
                bg.CancelAsync();
        }

        private void bg_DoWork(object sender, DoWorkEventArgs e){
            ICollection dr = (ICollection)e.Argument;
            names = new Dictionary<int, string>(10);
            names.Add(Session.Curr.User.ID, Session.Curr.User.Name);

            DataRow curr = null;
            int i = 0, p = 0;
            StreamWriter sw = null;
            try {
                sw = new StreamWriter(FileName, false, Encoding.UTF8);
                if (headers) sw.WriteLine(Headers());

                foreach(DataRow r in dr){
                    curr = r;
                    sw.WriteLine(format, 
                        Tester((int)r["uid"]),
                        SN(r["sn"]),
                        r["product"],
                        r["nr"],
                        Date(r["date"]),
                        TestData(r["vals"], r["comments"]),
                        ((string)r["comment"]).Trim("\n"[0], "\r"[0]).Replace("\"", "\\\""),
                        TestResult((TestState)r["state"], (int)r["result"])
                    );
                    
                    p = (int)(i * 100.0f / dr.Count);
                    if (p % 5 == 0)
                        bg.ReportProgress(p);

                    i++;
                }

                e.Result = dr.Count;
            }
            catch (ArgumentException exp){
                e.Result = String.Format("{0} dla pompy {1} o SN {2}", exp.Message, curr["product"], SN(curr["sn"], true));
            }
            catch (Exception exp) {
                Server.Log("Przerwano exportowanie danych na {0} - {1} w ({2})\n{3}", i, exp.GetType(), Dump(curr), exp.ToString());
                e.Result = String.Format("Nieoczekiwany błąd podczas Exportowania wyników testów dla pompy {0} o SN {1}",  curr["product"], SN(curr["sn"], true));
            }
            finally {
                if(sw != null)
                    sw.Close();
            }
        }

        private void bg_ProgressChanged(object sender, ProgressChangedEventArgs e){
            Session.Curr.Action("Zapisywanie... ("+e.ProgressPercentage+"%)");
        }

        private void bg_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e){
            if (e.Result is Int32)
                Session.Curr.Action("Pomyślnie zapisano " + e.Result + " rekordów w " + FileName);
            else {
                string msg = (e.Result is string) ? e.Result as string : "Nieznany błąd podczas Exportowania"; 
                Session.Curr.Action("Nie udalo się zapisać do pliku " + FileName);
                MessageBox.Show(msg, "Błąd podczas exportu", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private string Tester(int uid) {
            string name = null;
            if (names.TryGetValue(uid, out name))
                return name;

            try { 
                name = (string)Server.Command("SELECT name FROM tapflodb.user WHERE id = " + uid).ExecuteScalar();
            }
            catch (Exception exp){ 
                Server.Log("Podczas pobierania nazwy użytkownika od id {0} wustąpił nieoczekiwany bład - {1}", uid, exp.Message);
                name = "(Nieznany)";
            }
            names.Add(uid, name);

            return name;
        }

        private string Date(object o) {
            if(o is DateTime)
                return ((DateTime)o).ToString("dd.MM.yyyy HH:mm:ss");
            return "";
        }

        private string TestData(object v, object c) {
            int len = TDB.TestDef.Rows.Count;
            int?[] vals = PSqlArrays.ParseInt(v, len);
            string[] comm = PSqlArrays.ParseString(c, len);
            bool emptyfirst = false;
            String tmp;
            TestType tt;
            StringBuilder sb = new StringBuilder(vals.Length * 10);
            for (int i = 0; i < len; i++){
                if (emptyfirst || sb.Length > 0) sb.Append(separator);
                
                tt = (TestType)TDB.TestDef.Rows[i]["type"];
                if(tt == TestType.Unknown)
                    continue;
                else if(tt == TestType.PassFail){
                    if (i >= vals.Length || !vals[i].HasValue){
                        sb.AppendFormat("{0}", separator);
                        emptyfirst = sb.Length == 0;
                    }
                    else if (vals[i].Value == 0)
                        sb.AppendFormat("\"{1}\"{0}", separator, Faults.Fault.Pass); // Pass,, //Empty Value
                    else
                        sb.AppendFormat("\"{1}\"{0}\"{2}\"", separator, Faults.Fault.Fail, Faults.Fault.Find(vals[i].Value));
                }
                else {
                    // Skip to next if no data
                    if (i >= vals.Length || !vals[i].HasValue){
                        emptyfirst = sb.Length == 0;
                        continue;
                    }

                    tmp = (new Decimal(vals[i].Value) / 1000).ToString("0.0", CultureInfo.InvariantCulture);
                    if(quotes)
                        sb.AppendFormat("\"{0}\"", tmp.Replace(".", decsep));
                    else
                        sb.Append(tmp.Replace(".", decsep));
                }
            }

            return sb.ToString();
        }

        private string TestResult(TestState ts, int id) {
            if (ts.IsSet(TestState.Passed))
                return String.Format("\"{1}\"{0}\"\"", separator, Faults.Fault.Pass.Name);
            else if (ts.IsSet(TestState.MechanicalDamage))
                return String.Format("\"{1}\"{0}\"\"", separator, Faults.Fault.Damage.Name);
            else if (id < 0 || id >= TDB.Tests.Count){
                Server.Log("Nieznany id testu {0} / {1}", id, TDB.Tests.Count);
                throw new ArgumentException("Nieznana przyczyna Końcowego Niezaliczenia testów");
            }

            return String.Format("\"{1}\"{0}\"{2}\"", separator, Faults.Fault.Fail.Name, TDB.Tests[id]);
        }

        private string Headers() {
            TestType tt;
            StringBuilder sb = new StringBuilder(200);
            sb.AppendFormat("\"Tester\"{0}\"SN\"{0}\"Produkt\"{0}\"Literacja\"{0}\"Data\"{0}", separator);
            for (int i = 0; i < TDB.TestDef.Rows.Count; i++){
                sb.AppendFormat("\"{1}\"{0}", separator, TDB.TestDef.Rows[i]["name"]);

                tt = (TestType)TDB.TestDef.Rows[i]["type"];
                if(tt == TestType.Unknown)
                    continue;
                else if(tt == TestType.PassFail)
                    sb.AppendFormat("\"{1} Defekt\"{0}", separator, TDB.TestDef.Rows[i]["name"]);
            }
            sb.AppendFormat("\"Komentarz\"{0}\"Rezultat\"{0}\"Przyczyna\"", separator);

            return sb.ToString();
        }

        private string SN(object obj, bool safe = false) {
            int sn;
            if(!Int32.TryParse(obj.ToString(), out sn)){
                if (safe)
                    throw new ArgumentException("Błędny numer seryjny produktu");
                else
                    return obj.ToString();
            }

            return sn.ToSN();
        }

        private string Dump(DataRow r) {
            if (r == null)
                return "(brak)";
            return String.Join(";", new String[] {
                        "UID=" + r["uid"],
                        "SN=" + r["sn"],
                        r["product"].ToString(),
                        r["nr"].ToString(),
                        r["date"].ToString(),
                        r["vals"].ToString(),
                        r["comments"].ToString(),
                        r["comment"].ToString(),
                        r["state"].ToString(),
                        r["result"].ToString()
            });
        }
    }
}
