﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.Text;
using System.Configuration;

namespace TapfloDB.View.TDB
{
    class LabelPrinter : System.Drawing.Printing.PrintDocument
    {
        #region  Properties

        public static Font ModelFont { get; set; }
        public static Font SNFont { get; set; }

        public static RectangleF ModelRect { get; set; }
        public static RectangleF SNRect { get; set; }

        public string Model { get; set; }
        public string SN { get; set; }
        
        #endregion

        #region  Class Constructors 

        static LabelPrinter() {
            ModelFont = new Font("Arial", 18f, FontStyle.Bold);
            SNFont = new Font("Arial", 14f, FontStyle.Bold);

            //First Try
            //ModelRect = new RectangleF(88.38f, 18.12f, 72f, 8f);
            //SNRect = new RectangleF(107.43f, 29.43f, 52f, 7f);
            //Adjusted
            //ModelRect = new RectangleF(79.38f, 12.12f, 72f, 8f);
            //SNRect = new RectangleF(98.43f, 23.43f, 52f, 7f);
            //2014 Update
            ModelRect = new RectangleF(79.5f, 12f, 72f, 8f);
            SNRect = new RectangleF(98.5f, 24.5f, 52f, 7f);

            LoadSettings();
        }

        /// <summary>
        /// Empty constructor
        /// </summary>
        /// <remarks></remarks>
        public LabelPrinter() : base(){
        }

        /// <summary>
        /// Constructor to initialize our printing object
        /// and the text it's supposed to be printing
        /// </summary>
        /// <param name=str>Text that will be printed</param>
        /// <remarks></remarks>
        public LabelPrinter(string model, string sn) : this(){
            Model = model;
            SN = sn;
        }

        internal static void LoadSettings() {
            float x = ModelRect.X, y = ModelRect.Y;
            if (ConfigurationManager.AppSettings["ModelX"] != null)
                float.TryParse(ConfigurationManager.AppSettings["ModelX"], out x);
            if (ConfigurationManager.AppSettings["ModelY"] != null)
                float.TryParse(ConfigurationManager.AppSettings["ModelY"], out y);
            ModelRect = new RectangleF(x, y, ModelRect.Width, ModelRect.Height);

            x = SNRect.X; y = SNRect.Y;
            if (ConfigurationManager.AppSettings["SNY"] != null)
                float.TryParse(ConfigurationManager.AppSettings["SNX"], out x);
            if (ConfigurationManager.AppSettings["SNY"] != null)
                float.TryParse(ConfigurationManager.AppSettings["SNY"], out y);
            SNRect = new RectangleF(x, y, SNRect.Width, SNRect.Height);
        }

        #endregion

        #region Print function

        protected override void OnBeginPrint(PrintEventArgs e){
            base.OnBeginPrint(e);

            DocumentName = "Etykieta " + Model;
        }

        /// <summary>
        /// Override the default OnPrintPage method of the PrintDocument
        /// </summary>
        /// <param name=e></param>
        /// <remarks>This provides the print logic for our document</remarks>
        protected override void OnPrintPage(System.Drawing.Printing.PrintPageEventArgs e)
        {
            // Run base code
            base.OnPrintPage(e);

            //Declare local variables needed
            /*
            int printHeight;
            int printWidth;
            int leftMargin;
            int topMargin;

            printHeight = base.DefaultPageSettings.PaperSize.Height - base.DefaultPageSettings.Margins.Bottom;
            printWidth = base.DefaultPageSettings.PaperSize.Width - base.DefaultPageSettings.Margins.Right;
                
            leftMargin = base.DefaultPageSettings.Margins.Left;
            topMargin = base.DefaultPageSettings.Margins.Top;

            //Check if the user selected to print in Landscape mode
            //if they did then we need to swap height/width parameters
            if (base.DefaultPageSettings.Landscape){
                int tmp;
                tmp = printHeight;
                printHeight = printWidth;
                printWidth = tmp;
            }
            */
            //Now we need to determine the total number of lines
            //we're going to be printing
            //Int32 numLines = (int)printHeight / PrinterFont.Height;

            //Use the StringFormat class for the text layout of our document
            StringFormat format = new StringFormat(StringFormatFlags.LineLimit);

            //Fit as many characters as we can into the print area      
            //e.Graphics.MeasureString(_text.Substring(RemoveZeros(curChar)), PrinterFont, new SizeF(printWidth, printHeight), format, out chars, out lines);

            e.Graphics.PageUnit = GraphicsUnit.Millimeter;
            //Print the page
            e.Graphics.DrawString(Model, ModelFont, Brushes.Black, ModelRect, format);
            e.Graphics.DrawString(SN, SNFont, Brushes.Black, SNRect, format);

            e.HasMorePages = false;
        }

        #endregion
    }
}
