﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Npgsql;
using NpgsqlTypes;
using System.IO;
using System.Collections;
using ExcelLibrary;
using ExcelLibrary.SpreadSheet;

namespace TapfloDB.View.TDB
{
    public partial class Main : AbstractView
    {
        private string where = "";
        private DataTable dt = new DataTable();

        private string[] addopt = {"Testuj po raz kolejny pompę {0} (SN: {1})",
                                   "Testuj kolejną pompę {0} do zamówienia nr {1}",
                                   "Testuj kolejną pompę z zamówienia nr {0}"};
        private string[] printopt = {"Ustawienia Wydruku"};

        public Main(){
            InitializeComponent();
            SupportExport = true;
            Text = "Test Database";

            dgv.AutoGenerateColumns = false;
            
            //Last two months filter calculation
            int y = DateTime.Now.Year, m = DateTime.Now.Month - 1;
            if (m < 1) { y--; m = 12; }
            int sub = DateTime.Now.Day + DateTime.DaysInMonth(y, m) - 1;
            date.From.Value = DateTime.Today.Subtract(TimeSpan.FromDays(sub));
            date.To.Value = DateTime.Today.AddSeconds(1440 * 60 - 1);

            date.From.ValueChanged += new EventHandler(date_ValueChanged);
            date.To.ValueChanged += new EventHandler(date_ValueChanged);

            badd.DropDownFont = badd.Font;
            blabels.DropDownFont = blabels.Font;
            blabels.AddDropDownItemAndHandle(printopt[0], new EventHandler(blpo_Click));
        }

        void date_ValueChanged(object sender, EventArgs e){
            if(date.From.Value <= date.To.Value){
                Query(FilterType.Date);
            }
        }

        private void Main_Load(object sender, EventArgs e){
            Server.Connect();

            Data.User u = Session.Curr.User;

            //Filter Menus
            if (u.Type != Users.UserType.Admin){
                bdelete.Visible = false;
                badd.Location = bedit.Location;
                bedit.Location = bdelete.Location;
            }

            Query(FilterType.Date); //Default by Date
            dgv.DataSource = dt;
        }

        #region Filter

        private void tsearch_KeyUp(object sender, KeyEventArgs e){
            if (e.KeyCode == Keys.Return){
                bsearch_Click(sender, e);
            }
        }

        private void tsearch_TextChanged(object sender, EventArgs e){
            if (tsearch.Text.Length == 0)
                Query(FilterType.Date);
        }

        private void bsearch_Click(object sender, EventArgs e){
            int i = Query(FilterType.Query);
            if (i > 0)
                Session.Curr.Action("Znaleziono " + i + " pasujących wpisów dla \"" + tsearch.Text + "\"");
            else
                Session.Curr.Action("Nie znaleziono pasujących wpisów");
        }

        private string fields = "SELECT sn, nr, product, orderid, uid, date, vals::text, comments, result, state, comment";
        private int Query(FilterType filter){
            if (filter == FilterType.Unknown) return -1;
            
            int d = 0;
            string q = tsearch.Text.Trim();
            if (filter == FilterType.None)
                where = " LIMIT 100";
            else if (filter == FilterType.Query && q.Length > 0){ //Text Search
                bool num = Int32.TryParse(q, out d);
                if (num && q.Length < 7)
                    where = String.Format(" WHERE sn::text ILIKE '{0}%' OR orderid::text ILIKE '{0}%';", d);
                else if (num && q.Length == 7)
                    where = String.Format(" WHERE orderid = {0};", d);
                else if (num && q.Length >= 8)
                    where = String.Format(" WHERE sn = {0};", d);
                else if (q.IsSN(ref d))
                    where = String.Format(" WHERE sn = {0};", d);
                else
                    where = String.Format(" WHERE product ILIKE '{0}%'", tsearch.Text);
            }
            else if(filter == FilterType.Date){ //By date
                where = String.Format(" WHERE date >= '{0}' AND date <= '{1}';", date.From.Value, date.To.Value);
            }

            NpgsqlDataAdapter da = Server.DataAdapter(fields + " FROM tdb.\"TestDatabase\"" + where);
            try {
                dt.Clear();
                return da.Fill(dt);
            }
            catch (Exception exp){
                Server.Log(filter != FilterType.None ? "Błąd filtrowania TDB {0} - {1}" : "Błąd listowania TDB - {1}", where, exp.Message);
                return -1;
            }
            finally {
                if (da != null)
                    da.Dispose();
            }
        }

        #endregion

        #region Data Column Formating

        private void dgv_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e){
            if (e.ColumnIndex < 2) return;
            else if (e.ColumnIndex == 2)
                e.Value = ((int)e.Value).ToSN();
            else if (e.ColumnIndex == 4)
                e.Value = ((int)e.Value).ToString("D7");
            else if (e.ColumnIndex > 5){
                DataRowView dr = dgv.Rows[e.RowIndex].DataBoundItem as DataRowView;
                TestState ts = (TestState)dr.Row["state"];
            
                // Rezultat
                if (e.ColumnIndex == 6){
                    if (ts.IsSet(TestState.Passed)){
                        e.Value = Faults.Fault.Pass.Name;
                        e.CellStyle.ForeColor = Color.Green;
                    }
                    else if (ts.IsSet(TestState.MechanicalDamage)){
                        e.Value = Faults.Fault.Damage.Name;
                        e.CellStyle.ForeColor = Color.Orange;
                    }
                    else {
                        e.Value = Faults.Fault.Fail.Name;
                        e.CellStyle.ForeColor = Color.Red;
                    }
                }
                // Przyczyna
                else if (e.ColumnIndex == 7){
                    if (ts.IsSet(TestState.Passed | TestState.MechanicalDamage))
                        e.Value = "(brak)";
                    else if ((int)e.Value < 0)
                        e.Value = "(Inna)";
                    else
                        e.Value = TDB.Tests[(int)e.Value];
                }
                /* Stan
                else if (e.ColumnIndex == 8){
                    if (ts.IsSet(TestState.Accepted)){
                        e.Value = "Zatwierdzony";
                        e.CellStyle.ForeColor = Color.Green;
                    }
                    else {
                        e.Value = "Do Zatwierdzenia";
                        e.CellStyle.ForeColor = Color.Red;
                    }
                }
                */
            }
        }

        #endregion

        #region Add

        private void badd_Click(object sender, EventArgs e){
            Test_ShowDialog(new Test());
        }

        private void baddNext_Click(object sender, EventArgs e){
            Test t = new Test();
            DataGridViewRow r = dgv.SelectedRows[0];
            t.SN = (int)r.Cells[2].Value;
            t.Product = (string)r.Cells[3].Value;
            t.OrderId = (int)r.Cells[4].Value;
            t.Nr = (short)(((short)r.Cells[5].Value) + 1);

            Test_ShowDialog(t);
        }

        private void baddProductOrder_Click(object sender, EventArgs e){
            Test t = new Test();
            DataGridViewRow r = dgv.SelectedRows[0];
            t.Product = (string)r.Cells[3].Value;
            t.OrderId = (int)r.Cells[4].Value;
            
            Test_ShowDialog(t);
        }

        private void baddOrder_Click(object sender, EventArgs e){
            Test t = new Test();
            DataGridViewRow r = dgv.SelectedRows[0];
            t.OrderId = (int)r.Cells[4].Value;

            Test_ShowDialog(t);
        }

        private void Test_ShowDialog(Test t){
            if (t.ShowDialog() == DialogResult.OK){
                Server.Connect();

                NpgsqlCommand cmd = Server.Command("INSERT INTO tdb.\"TestDatabase\" VALUES " +
                    "(:sn, :nr, :product, :orderid, :uid, :date, :vals, :comments, :result, :state, :comment);");
                cmd.Parameters.AddWithValue("sn", t.SN);
                cmd.Parameters.AddWithValue("nr", t.Nr);
                cmd.Parameters.AddWithValue("product", t.Product);
                cmd.Parameters.AddWithValue("orderid", t.OrderId);
                cmd.Parameters.AddWithValue("uid", Session.Curr.User.ID);
                cmd.Parameters.AddWithValue("date", DateTime.Now);
                cmd.Parameters.AddWithValue("result", (int)t.Result);
                cmd.Parameters.AddWithValue("state", (int)t.State);
                cmd.Parameters.AddWithValue("comment", t.Comment);
                cmd.Parameters.Add(new NpgsqlParameter("vals", NpgsqlDbType.Integer | NpgsqlDbType.Array));
                cmd.Parameters.Add(new NpgsqlParameter("comments", NpgsqlDbType.Text | NpgsqlDbType.Array));
                
                //Set params
                cmd.Parameters["vals"].Value = t.Vals;
                cmd.Parameters["comments"].Value = t.Comments;

                try {
                    if (cmd.ExecuteNonQuery() == 1){
                        Session.Curr.Action("Dodano nowy wpis do Test Database");

                        Query(FilterType.Refresh);
                    }
                }
                catch (Exception exp){
                    Server.Log("Błąd dodawania testu - {0}", exp.Message);
                    MessageBox.Show("Podczas dodawania wpisu do Test Database wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd dodawania wyników Testów", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
        }

        private void dgv_SelectionChanged(object sender, EventArgs e){
            if(dgv.SelectedRows.Count < 1) return; 
            badd.ClearDropDownItems();

            DataGridViewRow r = dgv.SelectedRows[0];
            string sn = ((int)r.Cells[2].Value).ToSN();
            string order = ((int)r.Cells[4].Value).ToOrder();
            badd.AddDropDownItemAndHandle(String.Format(addopt[0], r.Cells[3].Value, sn), new EventHandler(baddNext_Click));
            badd.AddDropDownItemAndHandle(String.Format(addopt[1], r.Cells[3].Value, order), new EventHandler(baddProductOrder_Click));
            badd.AddDropDownItemAndHandle(String.Format(addopt[2], order), new EventHandler(baddOrder_Click));
        }

        #endregion

        private void bedit_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count == 0)
                return;

            DataRowView drv = dgv.SelectedRows[0].DataBoundItem as DataRowView;
            Test t = new Test(drv.Row);
            
            //Check user access
            Data.User u = Session.Curr.User;
            if (u.Type != Users.UserType.Admin) {
                if (u.ID != (int)drv.Row["uid"]){
                    MessageBox.Show("Nie możesz edytować tego wpisu!", "Odmowa dostępu", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                else if (((DateTime)drv.Row["date"]).AddDays(1) < DateTime.Now){
                    MessageBox.Show("Nie możesz edytować tego wpisu!", "Odmowa dostępu", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }

            int osn = t.SN;
            short onr = t.Nr;
            if (t.ShowDialog() == DialogResult.OK){
                Server.Connect();

                NpgsqlCommand cmd = Server.Command("UPDATE tdb.\"TestDatabase\" SET " +
                    Server.SetClause("sn", "nr", "product", "orderid", /* "uid", */ "vals", "comments", "result", "state", "comment") +
                    " WHERE sn = :osn AND nr = :onr;");
                cmd.Parameters.AddWithValue("osn", osn);
                cmd.Parameters.AddWithValue("onr", onr);
                cmd.Parameters.AddWithValue("sn", t.SN);
                cmd.Parameters.AddWithValue("nr", t.Nr);
                cmd.Parameters.AddWithValue("product", t.Product);
                cmd.Parameters.AddWithValue("orderid", t.OrderId);
                //cmd.Parameters.AddWithValue("uid", u.ID);
                cmd.Parameters.AddWithValue("result", (int)t.Result);
                cmd.Parameters.AddWithValue("state", (int)t.State);
                cmd.Parameters.AddWithValue("comment", t.Comment);
                cmd.Parameters.Add(new NpgsqlParameter("vals", NpgsqlDbType.Integer | NpgsqlDbType.Array));
                cmd.Parameters.Add(new NpgsqlParameter("comments", NpgsqlDbType.Text | NpgsqlDbType.Array));

                //Set params
                cmd.Parameters["vals"].Value = t.Vals;
                cmd.Parameters["comments"].Value = t.Comments;

                try {
                    if (cmd.ExecuteNonQuery() == 1){
                        Session.Curr.Action("Zapisano zmiany w Test Database");

                        Query(FilterType.Refresh);
                    }
                }
                catch (Exception exp){
                    Server.Log("Błąd edycji testu dla #{0}/{1} - {2}", osn, onr, exp.Message);
                    MessageBox.Show("Podczas zmiany wpisu w Test Database wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd dodawania Użytkownika", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
            }
        }

        private void bdelete_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count == 0)
                return;

            if (Question("Czy na pewno skasować", "Potwierdz usunięcie") == DialogResult.Yes){
                int i = 0;
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (TDB.Delete((int)r.Cells["csn"].Value, (short)r.Cells["cnr"].Value))
                        i++;
                }

                if (i > 0){
                    Session.Curr.Action("Skasowano " + i + " wpisów z Test Database");
                    Query(FilterType.Refresh);
                }
                else
                    Session.Curr.Action("Nie można skasować wybranych wpisów");
            }
        }

        /* Save Button
        private void bsave_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count == 0)
                return;

            if (Question("Czy na pewno zatwierdzić", "Potwierdz zatwierdzenie") == DialogResult.Yes){
                int i = 0;
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    if (TDB.Accept((int)r.Cells["csn"].Value, (short)r.Cells["cnr"].Value))
                        i++;
                }

                if (i > 0){
                    Session.Curr.Action("Zatwierdzono " + i + " wpisów w Test Database");
                    Query(true);
                }
                else
                    Session.Curr.Action("Nie można zatwierdzić wybranych wpisów");
            }
        }
        */

        #region Refresh

        private void Main_KeyUp(object sender, KeyEventArgs e){
            if (e.KeyCode == Keys.F5)
                brefresh_Click(sender, e);
        }

        private void brefresh_Click(object sender, EventArgs e){
            if (Query(FilterType.Refresh) > 0)
                Session.Curr.Action("Pomyślnie odświerzono liste wykonanych Testów");
            else
                Session.Curr.Action("Nie ma nic do wyświetlenia");
        }

        #endregion

        private DialogResult Question(string text, string caption){
            int i = 0, id = Session.Curr.User.ID;
            StringBuilder sb = new StringBuilder();
            foreach (DataGridViewRow r in dgv.SelectedRows){
                if (i++ > 0) sb.Append(",\n");
                sb.AppendFormat("{0} (SN: {1}) Literacja {2}", r.Cells["cproduct"].Value, r.Cells["csn"].Value, r.Cells["cnr"].Value);
            }

            if (i == 0)
                return DialogResult.None;
            if (i == 1)
                text = String.Format("{0} \"{1}\"?", text, sb.ToString());
            else
                text = String.Format("{0} {1} serie testów?\n{2}", text, i, sb.ToString());

            return MessageBox.Show(text, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
        }

        public override void Export(){
            Server.Connect();

            NpgsqlCommand cmd = Server.Command("SELECT count(*) FROM tdb.\"TestDatabase\";");
            object o = cmd.ExecuteScalar();
            if(o is DBNull || (long)o == 0){
                MessageBox.Show("Nie znaleziono danych do wyeksportowania z Test Database.", "Test Database jest pusta", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            long a = (long)o;
            Export e = new Export(dgv.SelectedRows.Count, dgv.Rows.Count, (int)a);
            if (e.ShowDialog() == DialogResult.OK) {
                if (e.Mode == 1)
                    e.Save((from DataGridViewRow r in dgv.SelectedRows
                            select ((DataRowView)r.DataBoundItem).Row).ToArray());
                else if (e.Mode == 2)
                    e.Save(dt.Rows);
                else if (e.Mode == 3){
                    DataTable all = new DataTable();
                    NpgsqlDataAdapter da = Server.DataAdapter(fields + " FROM tdb.\"TestDatabase\";");
                    da.Fill(all);
                    e.Save(all.Rows);
                }
            }
        }

        #region Dump

        private void bdump_Click(object sender, EventArgs e){
            if (bg.IsBusy) return;
            if (dgv.SelectedRows.Count < 1) {
                MessageBox.Show("Nie zaznaczono żadnych rekordów do zapisania!", "Brak danych do zapisania", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            if (save.ShowDialog() == DialogResult.OK) {
                object[] args = new object[2];
                args[0] = save.FileName;
                args[1] = (from DataGridViewRow r in dgv.SelectedRows
                           select ((DataRowView)r.DataBoundItem).Row).ToArray();
                
                if(save.FilterIndex == 1)
                    bg.DoWork += new DoWorkEventHandler(this.XLS_Worker);
                else
                    bg.DoWork += new DoWorkEventHandler(this.CSV_Worker);
            
                bg.RunWorkerAsync(args);
            }
        }

        private void CSV_Worker(object sender, DoWorkEventArgs e){
            object[] args = (object[])e.Argument;
            String name = (String)args[0];
            ICollection dr = (ICollection)args[1];
            
            StreamWriter sw = null;
            int i = 0, p = 0;
            try {
                sw = new StreamWriter(name, false, Encoding.UTF8);
                
                foreach (DataRow r in dr){
                    sw.WriteLine("\"{0}\";\"{1}\";\"{2}\"",
                        ((int)r["sn"]).ToSN(),
                        r["product"],
                        ((int)r["orderid"]).ToOrder()
                    );

                    p = (int)(i * 100.0f / dr.Count);
                    if (p % 5 == 0)
                        bg.ReportProgress(p);

                    i++;
                }

                e.Result = dr.Count;
            }
            catch (Exception exp){
                Server.Log("Błąd zapisywania dla Magazynu {0}/{1} - {21}", i, dr.Count, exp.Message);
                e.Result = -1;
            }
            finally {
                if (sw != null)
                    sw.Close();
            }
        }

        private void XLS_Worker(object sender, DoWorkEventArgs e){
            object[] args = (object[])e.Argument;
            String name = (String)args[0];
            ICollection dr = (ICollection)args[1];

            Workbook wb = new Workbook();
            int i = 0, p = 0;
            try {
                Worksheet ws = new Worksheet("TDB " + DateTime.Now.ToShortDateString());
    
                foreach (DataRow r in dr){
                    ws.Cells[i, 0] = new Cell(((int)r["sn"]).ToSN());
                    ws.Cells[i, 1] = new Cell(r["product"]);
                    //ws.Cells[i, 2] = new Cell(((int)r["orderid"]).ToOrder(), CellFormat.General);
                    ws.Cells[i, 2] = new Cell((int)r["orderid"]);
                    
                    p = (int)(i * 100.0f / dr.Count);
                    if (p % 5 == 0)
                        bg.ReportProgress(p);

                    i++;
                }

                wb.Worksheets.Add(ws);
                wb.Save(name);

                e.Result = dr.Count;
            }
            catch (Exception exp){
                Server.Log("Błąd zapisywania dla Magazynu {0}/{1} - {21}", i, dr.Count, exp.Message);
                e.Result = -1;
            }
        }

        private void bg_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e){
            int c = (int)e.Result;
            if (c < 0)
                Session.Curr.Action("Nie udalo się zapisać do pliku " + save.FileName);
            else
                Session.Curr.Action("Pomyślnie zapisano " + c + " rekordów w " + save.FileName);
        }

        #endregion

        #region Printing

        private void blabels_Click(object sender, EventArgs e){
            if (dgv.SelectedRows.Count < 1){
                MessageBox.Show("Nie zaznaczono żadnych rekordów do wydrukowanie etykiet!", "Brak danych do drukowania etykiet", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

            LabelPrinter printer = new LabelPrinter();
            Server.Log("Drukowanie na {0} \n{1} \n{2}", printer.PrinterSettings.PrinterName, printer.PrinterSettings.ToString(), printer.DefaultPageSettings.ToString());

            try {
                foreach (DataGridViewRow r in dgv.SelectedRows){
                    printer = new LabelPrinter((string)r.Cells["cproduct"].Value, ((int)r.Cells["csn"].Value).ToSN());
                    Session.Curr.Action("Drukowanie etykiety dla " + printer.Model + "...");
                    printer.Print();
                }
                Session.Curr.Action("Pomyslnie wydrukowano " + dgv.SelectedRows.Count + " etykiet");
            }
            catch (Exception exp) {
                Server.Log("Bład podczas drukowania etykiet - {0}", exp.Message);
            }
        }

        private void blpo_Click(object sender, EventArgs e){
            LabelSettings ls = new LabelSettings();
            ls.ShowDialog();
        }

        #endregion
    }
}
