﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TapfloDB.View.TDB
{
    public partial class Test : Form
    {
        #region Fields

        private TestRow[] tests;

        private bool existing;
        public bool Edited { get; set; }
        
        public int?[] Vals { get; private set; }
        public string[] Comments { get; private set; }
        public int Result { get; private set; }
        public TestState State { get; private set; }

        #endregion

        #region Properties

        public int SN {
            get {
                int sn = -1;
                Int32.TryParse(tsn.Text.Replace("-", ""), out sn);
                return sn; 
            }
            set { tsn.Text = value.ToSN(); Edited = true; } 
        }
        
        public string Product {
            get { return tproduct.Text; }
            set { tproduct.Text = value; Edited = true; }
        }

        public int OrderId {
            get {
                int id = -1;
                Int32.TryParse(torder.Text, out id);
                return id;
            }
            set { torder.Text = value.ToOrder(); Edited = true; } 
        }
        
        public short Nr {
            get { return (short)tnr.Value; }
            set { tnr.Value = (decimal)value; Edited = true; }
        }
        
        public string Comment {
            get { return tcomment.Text; }
            set { tcomment.Text = value; Edited = true; }
        }

        #endregion

        #region Constructor

        public Test(){
            InitializeComponent();
            cresult.DataSource = Faults.Fault.FinalResult;
            creason.DataSource = TDB.Tests;
            existing = false;

            tests = TestRow.Build();
            Vals = new int?[tests.Length];
            Comments = new String[tests.Length];
            State = TestState.Unknown;
            Result = -1;

            //Final Results
            cresult.SelectedIndex = -1;
            creason.SelectedIndex = -1;

            InitTest();
        }
        public Test(DataRow dr){
            InitializeComponent();
            cresult.DataSource = Faults.Fault.FinalResult;
            creason.DataSource = TDB.Tests;
            existing = true;

            SN = (int)dr["sn"];
            Product = (string)dr["product"];
            OrderId = (int)dr["orderid"];
            Nr = (short)dr["nr"];

            Vals = PSqlArrays.ParseInt((string)dr["vals"], TDB.TestDef.Rows.Count);
            Comments = PSqlArrays.ParseString(dr["comments"], TDB.TestDef.Rows.Count);
            tests = TestRow.Build(Vals, Comments);
            
            State = (TestState)dr["state"];
            Result = (int)dr["result"];
            if (State.IsSet(TestState.Passed)){
                cresult.SelectedIndex = 0;
                creason.SelectedIndex = -1;
            }
            else if (State.IsSet(TestState.MechanicalDamage)){
                cresult.SelectedIndex = 2;
                creason.SelectedIndex = -1;
            }
            else {
                cresult.SelectedIndex = 1;
                creason.SelectedIndex = (Result > 0) ? Result : -1;
            }

            Comment = (string)dr["comment"];
            
            InitTest();
        }

        public void InitTest(){
            Edited = false;
            dgv.Rows.AddRange(tests);
            
            for (int i = 0; i < tests.Length; i++)
                dgv_CellValueChanged(this, new DataGridViewCellEventArgs(1, i));
        }

        #endregion

        #region Save Button

        private void bsave_Click(object sender, EventArgs e){
            string msg;
            if (tsn.Text.Length != 9 || tsn.Text.IndexOf('-') != 4 || SN < 0){
                MessageBox.Show("Podany numer seryjny Produktu jest błędny, wymagany format \"rrmm-xxxx\"", "Błędny numer seryjny", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tsn.Focus();
                return;
            }

            if (tproduct.Text.Length == 0){
                MessageBox.Show("Nie podano nazwy Produktu (minimum 3 znaki)", "Nie podano nazwy Produktu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tproduct.Focus();
                return;
            }

            if (tproduct.Text.Length < 3){
                MessageBox.Show("Podana nazwa Produktu jest zbyt krótka (minimum 3 znaki)", "Za krótka nazwa Produktu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                tproduct.Focus();
                return;
            }

            if (torder.Text.Length != 7 || OrderId < 0){
                MessageBox.Show("Nr zlecenia musi składać się dokładnie z 7 cyfr.", "Błędny nr zlecenia", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                torder.Focus();
                return;
            }
            
            int lnr = TDB.LastIter(SN);
            if (!existing && Nr != lnr + 1){
                msg = String.Format("Produkt o numerze seryjnym {0} przeszedł {3} literacje nr {1}.\nCzy chcesz ustawić kolejną literacje (nr {2})?", tsn.Text, Nr, lnr + 1, Nr <= lnr ? "już" : "dopiero");
                if (MessageBox.Show(msg, "Błędny numer Literacji", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    Nr = (short)(lnr + 1);
                else {
                    tnr.Focus();
                    return;
                }
            }

            // find required tests rule for this Product
            TestRules.TestRule rule = TestRules.TestRule.Find(Product);

            int nc = 0; // null comments counter
            bool f, fail = false; //Has any fail in Pass/Fail Tests?
            string comm;
            foreach(TestRow r in tests){
                if (rule.IsRequired(r.Id) && r.Check() > 0){
                    MessageBox.Show("Nie wprowadzono wyniku dla testu \"" + r.Cells[0].Value + "\".", "Brak wyniku dla testu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    return;
                }

                // Pass / Fail
                if (r.Type == TestType.PassFail && r.Value != null){
                    f = r.Value != 0;
                    if (f && r.Value < 0) {
                        MessageBox.Show("Nie wybrano przyczyny niezaliczenia testu \"" + r.Cells[0].Value + "\".", "Brak przyczyny oblania testu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                        return;
                    }
                    fail |= f; // update
                }
                Vals[r.Index] = r.Value; //Set Value
                
                // Test Comment for null coments
                comm = r.Cells[3].Value as string;
                if(comm == null || comm.Trim().Length == 0){
                    Comments[r.Index] = "";
                    nc++;
                }
                else
                    Comments[r.Index] = comm.TrimStart().TrimEnd();
            }
            
            int finalresult = cresult.SelectedIndex;
            if (finalresult < 0){
                MessageBox.Show("Nie ustawiono końcowego rezultatu.", "Brak ostatecznego rezultatu", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                cresult.Focus();
                return;
            }

            if (fail && finalresult != Faults.Fault.Fail.ID){
                MessageBox.Show(String.Format("Końcowy rezultat nie może być \"{0}\" gdy któryś z testów został \"{1}\".", Faults.Fault.Pass, Faults.Fault.Fail), "Sprzeczne rezultaty", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                cresult.Focus();
                return;
            }

            if ((fail || finalresult == Faults.Fault.Fail.ID) && creason.SelectedIndex < 0){
                MessageBox.Show(String.Format("Gdy końcowy rezultat jest \"{0}\" musi zostać wybrana przyczyna!", Faults.Fault.Fail), "Nie wybrano przyczyny Niezaliczenia", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                creason.Focus();
                return;
            }

            // Clear existing flag
            State &= ~(TestState.Passed | TestState.MechanicalDamage);
            Result = 0;

            // Set final pass/fail/damage flag
            if (finalresult == Faults.Fault.Pass.ID){
                State |= TestState.Passed;
            }
            else if (finalresult == Faults.Fault.Fail.ID){
                // empty 'State' == Fail
                Result = creason.SelectedIndex;
            }
            else if (finalresult == Faults.Fault.Damage.ID){
                State |= TestState.MechanicalDamage;
            }
            
            // all coments are null?
            if (nc == Comments.Length)
                Comments = null;

            // close dialog
            DialogResult = DialogResult.OK;
        }

        #endregion

        #region Adjust GUI

        private void dgv_CellValueChanged(object sender, DataGridViewCellEventArgs e){
            if (e.RowIndex < 0) return;
            Edited = true;
            if (e.ColumnIndex == 1 && tests[e.RowIndex].Type == TestType.PassFail) {
                if (dgv[1, e.RowIndex].Value == null || (int)dgv[1, e.RowIndex].Value == 0){
                    dgv[2, e.RowIndex].Value = null;
                    dgv[2, e.RowIndex].ReadOnly = true;
                }
                else
                    dgv[2, e.RowIndex].ReadOnly = false;
            }
        }

        private void cresult_SelectedIndexChanged(object sender, EventArgs e){
            if (creason.Enabled = cresult.SelectedIndex == Faults.Fault.Fail.ID)
                creason.Focus();
            else
                creason.SelectedIndex = -1;
        }

        private void dgv_DataError(object sender, DataGridViewDataErrorEventArgs e){
            e.Cancel = true;
        }

        #endregion

        #region DGV Keyboard Navigation

        // Basic Keyboard manipulation on DGV level
        private void dgv_KeyDown(object sender, KeyEventArgs e){
            if (dgv.SelectedRows.Count < 0) return;
            TestRow r = dgv.SelectedRows[0] as TestRow;
            if (e.KeyCode == Keys.Escape)
                r.Cells[1].Value = null;

            // Quick Pass/Fail/Yes/No
            if (r.Type != TestType.PassFail) return;

            if(e.KeyCode == Keys.Y || e.KeyCode == Keys.T || e.KeyCode == Keys.Z || e.KeyCode == Keys.D5 || e.KeyCode == Keys.NumPad5)
                r.Cells[1].Value = 0;
            else if (e.KeyCode == Keys.N || e.KeyCode == Keys.D0 || e.KeyCode == Keys.NumPad0){
                r.Cells[1].Value = 1;
                dgv.CurrentCell = r.Cells[2]; // Move focus to fail reason
            }
        }

        // Adjust EditControls behavior
        private void dgv_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e){
            // Ignore Entering Comment column
            if (dgv.CurrentCell.ColumnIndex > 2) return; 

            TestRow r = dgv.CurrentRow as TestRow;
            if (r.Type == TestType.PassFail && dgv.CurrentCell.ColumnIndex == 1){
                ComboBox pf = e.Control as ComboBox;
                if (r.Value == null)
                    pf.SelectedIndex = -1;
                pf.KeyDown += new KeyEventHandler(pf_KeyDown);
            }
            if (r.Type == TestType.PassFail && dgv.CurrentCell.ColumnIndex == 2){
                ComboBox pf = e.Control as ComboBox;
                pf.KeyDown -= new KeyEventHandler(pf_KeyDown); // unregister old handler
                if (!r.Value.HasValue || r.Value.Value < 0)
                    pf.SelectedIndex = -1;
            }
        }

        // Keyboard manipulation on Pass/Fail test
        private void pf_KeyDown(object sender, KeyEventArgs e){
            TestRow r = dgv.SelectedRows[0] as TestRow;
            ComboBox pf = sender as ComboBox;

            // Only for Pass/Fail cels!!!
            if (e.KeyCode == Keys.Escape){
                pf.SelectedIndex = -1;
                r.Cells[1].Value = null; //index set not working after focus loss
            }
            else if (e.KeyCode == Keys.Y || e.KeyCode == Keys.T || e.KeyCode == Keys.Z || e.KeyCode == Keys.D5 || e.KeyCode == Keys.NumPad5){
                pf.SelectedIndex = 0;
                r.Cells[2].Value = null; //Reset fail reason if Pass
            }
            else if (e.KeyCode == Keys.N || e.KeyCode == Keys.D0 || e.KeyCode == Keys.NumPad0){
                pf.SelectedIndex = 1;
                e.SuppressKeyPress = true; // no pass further
                dgv.CurrentCell = r.Cells[2]; // Move focus to fail reason
            }
        }

        // Move column back on Pass/Fail reason selection completed
        private DataGridViewComboBoxCell lastReason = null;
        private void dgv_CellEndEdit(object sender, DataGridViewCellEventArgs e){
            TestRow r = dgv.SelectedRows[0] as TestRow;
            if (e.ColumnIndex == 2 && r.Type == TestType.PassFail){
                lastReason = dgv.CurrentCell as DataGridViewComboBoxCell;
            }
            else
                lastReason = null;
        }

        private void dgv_SelectionChanged(object sender, EventArgs e){
            if (lastReason != null && lastReason.RowIndex + 1 < dgv.Rows.Count){
                //SelectNextControl((Control)sender, true, true, true, true);
                dgv.CurrentCell = dgv.Rows[lastReason.RowIndex + 1].Cells[1];
            }
        }

        #endregion

        #region OPC/PLC

        private void bget_Click(object sender, EventArgs e){
            try {
                int t;
                bool now = true;
                StringBuilder dbg = new StringBuilder();
                StringBuilder err = new StringBuilder();

                global::OPC.Data.OPCItemState[] res = OPC.Conn.SyncRead();
                global::OPC.Data.OPCItemState s = null;
                for (int i = 0; i < res.Length; ++i) {
                    s = res[i];
                    if (s == null)
                        err.AppendFormat("#{0}: null\n",  i);
                    else if (s.Error > 0)
                        err.AppendFormat("#{0} - {1}: E = {2}\n", i, s.HandleClient, s.Error);
                    else {
                        try {
                            dbg.AppendFormat("{0}={1}({2}) Q={3} T={4}, ", s.HandleClient, s.DataValue, s.DataValue.GetType(), s.Quality, s.TimeStamp);
                            // handle Wanders
                            if (s.HandleClient == 1009){
                                now = !(bool)s.DataValue;
                                continue;
                            }
                            if (s.HandleClient == 9 && now)
                                continue;
                            
                            // Handle others
                            t = s.HandleClient - 1;
                            if (t >= 0 && t < tests.Length){
                                if (tests[t].Type == TestType.PassFail)
                                    tests[t].Value = ((bool)s.DataValue) ? 0 : -2;
                                else if(tests[t].Type == TestType.ValueRange)
                                    tests[t].Value = (int)(Double.Parse(s.DataValue.ToString()) * 1000);
                            }
                            else
                                err.AppendFormat("#{0}: Unknown Test: {1} = {2}, Q = {3}, T = {4}\n", i, s.HandleClient, s.DataValue, s.Quality, s.TimeStamp);
                        }
                        catch (Exception exp){
                            err.AppendFormat("{0} @ #{1}: {2} = {3}, Q = {4}, T = {5}\n", exp.GetType(), i, s.HandleClient, s.DataValue, s.Quality, s.TimeStamp);
                        }
                    }
                }

                State |= TestState.PLCData;
                if (dbg.Length > 0)
                    Server.Log("Debug data:\n{0}\n", dbg.ToString());
                if (err.Length > 0) {
                    Server.Log("Data Errors:\n{0}\n", err.ToString());
                    MessageBox.Show("Podczas pobierania danych wystapily bledy, sprawdz wprowadzone wartości w razie dalszych problemów skontaktuj sie z Administratorem.", "Błąd pobierania danych ze Stanowiska Testowego", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                    MessageBox.Show("Dane zostaly pobrane i wprowadzone do formularza", "Pobieranie zakończone sukcesem", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception exp) {
                string extra = exp.InnerException != null ? exp.InnerException.Message : null;
                Server.Log("Wystapil blad podczas łączenia z serverem OPC {0} - {1} {2}", exp.GetType(), exp.Message, extra);
                MessageBox.Show("Podczas łączenia ze Stanowiskiem Testowym wystąpił nieznany błąd.\nSpróbuj ponownie za kilka minut lub skontaktuj się z Administratorem.", "Nieznany Błąd łączenia ze Stanowiskiem Testowym", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }

        #endregion
    }
}
