﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TapfloDB.View.TDB
{
    public class TestRow : DataGridViewRow
    {
        #region Properties

        public int Id { get; private set; }
        public TestType Type { get; private set; }

        public int? Value {
            get {
                if (Cells[1].Value == null)
                    return null;
                if (Type == TestType.PassFail){
                    //if (Cells[1].Value == null) return -1;
                    if ((int)Cells[1].Value == 0) return 0;
                    else return Cells[2].Value == null ? -2 : (int)Cells[2].Value;
                }
                else
                    return (int)(Decimal.Parse((string)Cells[1].Value) * 1000);
            }
            set {
                if (Type == TestType.PassFail){
                    if (!value.HasValue || value.Value == -1){
                        Cells[1].Value = null;
                        Cells[2].Value = null;
                    }
                    else if (value.Value == -2){
                        Cells[1].Value = 1;
                        Cells[2].Value = null;
                    }
                    else if (value.Value == 0){
                        Cells[1].Value = 0;
                        Cells[2].Value = null;
                    }
                    else if (value.Value > 0){
                        Cells[1].Value = 1;
                        Cells[2].Value = value;
                    }
                }
                else
                    Cells[1].Value = value.HasValue ? (new Decimal(value.Value) / 1000).ToString() : null;
            }
        }

        public string Comment {
            get { return Cells[3].Value as string; }
            set { Cells[3].Value = value; }
        }

        #endregion

        #region Static

        public static TestRow[] Build(){
            return Build(null, null);
        }
        public static TestRow[] Build(int?[] vals, string[] comment){
            DataRow r;
            TestRow tr;
            List<TestRow> trs = new List<TestRow>(TDB.TestDef.Rows.Count);
            for (int i = 0; i < TDB.TestDef.Rows.Count; i++){
                r = TDB.TestDef.Rows[i];
                if ((short)r["type"] == 0)
                    continue;

                tr = new TestRow(r);
                if (vals != null && i < vals.Length)
                    tr.Value = vals[i];
                if (comment != null && i < comment.Length)
                    tr.Comment = comment[i] == "NULL" ? null : comment[i];

                trs.Add(tr);
            }

            return trs.ToArray();
        }

        #endregion

        public TestRow(DataRow r){
            Id = (int)r["id"];
            Type = (TestType)r["type"];
            Cells.Add(new DataGridViewTextBoxCell { Value = r["name"] });
            Cells[0].ReadOnly = true;

            short[] extra = (short[])r["extra"];
            if(Type == TestType.PassFail){
                Cells.AddRange(
                    PassFailCell(),
                    FaultCell(extra[0]),
                    new DataGridViewTextBoxCell()
                );
            }
            else if (Type == TestType.ValueRange){
                Cells.AddRange(
                    NumericUpDownCell(extra),
                    new DataGridViewTextBoxCell(),
                    new DataGridViewTextBoxCell()
                );
                Cells[2].ReadOnly = true;
            }
        }

        private DataGridViewComboBoxCell PassFailCell(){
            DataGridViewComboBoxCell cell = new DataGridViewComboBoxCell();
            cell.DataSource = Faults.Fault.PassFail;
            cell.DisplayMember = "Name";
            cell.ValueMember = "ID";
            return cell;
        }

        private DataGridViewComboBoxCell FaultCell(int faults){
            DataGridViewComboBoxCell cell = new DataGridViewComboBoxCell();
            cell.DataSource = Faults.Fault.AllFaults;
            cell.DisplayMember = "Name";
            cell.ValueMember = "ID";
            return cell;
        }

        private TapfloDB.UI.DataGridViewNumericUpDownCell NumericUpDownCell(short[] extra){
            TapfloDB.UI.DataGridViewNumericUpDownCell cell = new TapfloDB.UI.DataGridViewNumericUpDownCell();
            cell.DecimalPlaces = extra.Length > 4 ? extra[4] : 1;
            cell.Increment = cell.DecimalPlaces == 0 ? 1 : new Decimal(1.0f / (cell.DecimalPlaces * 10));
            cell.Minimum = decimal.Parse(String.Format("{0},{1}", extra[0], extra[1]));
            cell.Maximum = decimal.Parse(String.Format("{0},{1}", extra[2], extra[3]));
            return cell;
        }

        public int Check(){
            /*
            name = name.ToUpper();
            if (name.StartsWith("CT") && Id != 6){
                return -1; //Skip other tests
            }
            else if (name.StartsWith("PD") || name.StartsWith("DT")){
                if (Id != 6 && Id != 8)
                    return -1; // Skip other tests
            }
            else  if (Id == 8 || Id == 9) // Tłumienie/Wanders
                return -1; //Skip     
            */

            return Cells[1].Value == null ? 1 : 0;
        }
    }

    public enum TestType : short { 
        Unknown = 0,
        PassFail = 1,
        ValueRange = 2
    }
}
