﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Threading;

namespace Updater
{
    public class PGCmd
    {
#if DEBUG
        private static string pgdir = @"\pgAdmin III\1.16\";
#else
        private static string pgdir = @"\PostgreSQL\9.2\bin\";
#endif
        private static string psql = "psql.exe";
        private static string pgdump = "pg_dump.exe";

        public static string Host { get; set; }
        public static string Port { get; set; }
        public static string DB { get; set; }

        static PGCmd() {
            string dir = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + pgdir;
            psql = dir + psql;
            pgdump = dir + pgdump;

            Host = "localhost";
            Port = "5432";
            DB = "tapflo";
        }

        public static Process PSql(string user, string pass, string args){
            Process process = new Process();
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;
            startInfo.CreateNoWindow = false;
            startInfo.UseShellExecute = false;
            startInfo.RedirectStandardOutput = true;
            startInfo.FileName = psql;
            startInfo.Arguments = String.Format("-h {0} -p {1} -d {2} -U {3} {4}", Host, Port, DB, user, args);
            startInfo.EnvironmentVariables.Add("PGPASSWORD", pass); ;
            
            process.StartInfo = startInfo;
            process.Start();

            return process;
        }

        public static Process Dump(string user, string pass, string args){
            string np = null;
            Process process = new Process();
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;
            startInfo.CreateNoWindow = false;
            startInfo.UseShellExecute = false;
            startInfo.RedirectStandardOutput = true;
            if (pass != null)
                startInfo.EnvironmentVariables.Add("PGPASSWORD", pass);
            else
                np = " --no-password";
            startInfo.FileName = pgdump;
            startInfo.Arguments = String.Format("-h {0} -p {1} -U {2}{3} {4}", Host, Port, user, np, args);
            
            process.StartInfo = startInfo;
            process.Start();

            return process;
        }

        public static string PGVersion = null;
        public static bool TryLogin(string user, string pass){
            Process p = PSql(user, pass, String.Format("-c \"select version();\""));
            string o = p.StandardOutput.ReadToEnd();

            int i = 10;
            while (--i > 0){
                if (p.HasExited)
                    break;
                
                Thread.Sleep(50);
                p.Refresh();
            }

            if (!p.HasExited){ //Force Exit
                p.Kill();
                Thread.Sleep(100);
                p.Close();
                return false;
            }
            else if(p.ExitCode == 0){
                string[] tmp = o.Split('\n');
                PGVersion = tmp[2].Trim();
            }

            return p.ExitCode == 0;
        }

        private static Version appv = null;
        public static Version GetDBVersion(string user, string pass){
            if (appv != null) return appv;

            Process p = PSql(user, pass, String.Format("-c \"select tapflodb();\""));
            string o = p.StandardOutput.ReadToEnd();

            int i = 10;
            while (--i > 0){
                if (p.HasExited)
                    break;

                Thread.Sleep(50);
                p.Refresh();
            }

            if (!p.HasExited){ //Force Exit
                p.Kill();
                Thread.Sleep(100);
                p.Close();
                return null;
            }
            else if(p.ExitCode == 0){
                string[] tmp = o.Split('\n');
                appv = new Version(tmp[2].Trim());
            }

            return appv;
        }
    }
}
