﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Threading;

namespace Updater
{
    public partial class Updater : Form
    {
        private Version Version = new Version("1.7.0.0");
        private Logger logger;
        private string user;
        private string pass;
        private string dump;

        public Updater(){
            InitializeComponent();
            Logger.Box = llog;
            logger = Logger.Get();
        }

        private void Updater_Load(object sender, EventArgs e){
            if (ConfigurationManager.AppSettings["Host"] != null)
                PGCmd.Host = ConfigurationManager.AppSettings["Host"];
            if (ConfigurationManager.AppSettings["Port"] != null)
                PGCmd.Port = ConfigurationManager.AppSettings["Port"];
            if (ConfigurationManager.AppSettings["DB"] != null)
                PGCmd.DB = ConfigurationManager.AppSettings["DB"];

#if DEBUG
            user = "postgres";
            pass = "VBoxU12";
            if (PGCmd.TryLogin(user, pass))
                logger.Log("Debug login as " + user);
            else {
                logger.Log("Debug login failed " + user);
                Close();
                return;
            }
#else
            Login l = new Login();
            if (l.ShowDialog(this) != DialogResult.OK){
                Close();
                return;
            }
            logger.Log(String.Format("Pomyślnie Zalogowano jako {0} ({1} @ {2}:{3})", l.UserName, PGCmd.DB, PGCmd.Host, PGCmd.Port));
            
            user = l.UserName;
            pass = l.Password;
#endif
            
        }

        private void Updater_Shown(object sender, EventArgs e){
            bupdate_Click(sender, e);
        }

        private void bupdate_Click(object sender, EventArgs e){
            Version v = PGCmd.GetDBVersion(user, pass);
            logger.Log("Baza TapfloDB {0} ({1})", v == null ? "(Nieznana)" : v.ToString(), PGCmd.PGVersion);
            if (v != null){
                if (v == Version && MessageBox.Show("Baza została już zaktualizowana do wersji " + Version + ".\nCzy chcesz ponownie dokonać aktualizacji do tej wersji?", "Baza już zaktualizowana", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.No){
                    logger.Log("Przerwano.");
                    return;
                }
                else if (v > Version && MessageBox.Show("Baza jest już w nowszej ("+v+"). Uruchomienie aktualizacji do wersji  "+ Version + " może spowodować utratę danych.\nCzy mimo to chcesz uruchomić aktualizacji do tej wersji?", "Baza jest w nowszej wersji", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.No){
                    logger.Log("Przerwano.");
                    return;
                }
            }
            /*
            else if(MessageBox.Show("Nie można określić wersji bazy.", "Nieznana wersja Bazy") == System.Windows.Forms.DialogResult.No){
                logger.Log("Przerwano.");
                return;
            }
            */

            logger.Log("Rozpoczunam aktualizowanie do wersji {0}", Version);

            logger.Log("Tworzenie kopi zapasowej...");
            dump = DateTime.Now.ToString("ddMMyyyyHHmmss") + ".dump";
            string args = String.Format("--format plain --verbose --file \"{0}\" \"{1}\"", dump, PGCmd.DB);
            Process p = PGCmd.Dump(user, pass, args);
            p.WaitForExit();
            if (p.ExitCode == 0)
                logger.Log("Utworzono kopie zapasową ("+dump+")");
            else {
                logger.Log("Nie udało się utworzyć kopi zapasowej :(");
                return;
            }
            p.Close();

            String psqlout;
            logger.Log("Aktualizowanie struktury...");
            p = PGCmd.PSql(user, pass, String.Format("--host {0} --port {1} -d {2} -f script", PGCmd.Host, PGCmd.Port, PGCmd.DB));
            p.WaitForExit();
            int code = p.ExitCode;
            if (code == 0){
                logger.Log("Pomyślnie zaktualizowano");
                p.Close();
                
                logger.Log("Sprzątanie...");
                p = PGCmd.PSql(user, pass, String.Format("--host {0} --port {1} -d {2} -f clean", PGCmd.Host, PGCmd.Port, PGCmd.DB));
                p.WaitForExit();
                if (p.ExitCode == 0){
                    logger.Log("Zakończono");
                    psqlout = p.StandardOutput.ReadToEnd();
                    logger.Log(psqlout);
                }
                else
                    logger.Log("Należy ręcznie wykoać sprzątanie");
            }
            else {
                logger.Log("Nie udało się zaktualizować bazy :(");
                /*
                logger.Log("Przywracanie...");
                p = PGCmd.PSql(user, pass, String.Format("--host {0} --port {1} -d {2} -f {3}", PGCmd.Host, PGCmd.Port, PGCmd.DB, dump));
                p.WaitForExit();
                p.WaitForExit();
                if (p.ExitCode == 0)
                    logger.Log("Pomyślnie przywrócono");
                else
                    logger.Log("Nie udało się cofnąć zmian :(");
                */
                return;
            }
            p.Close();
        }
    }
}
